import re

from datetime import timedelta

import requests
from dateutil import parser


class LolaAPIParser(object):
    def __init__(self):
        self.default_check_in_time = '16:00'
        self.default_check_out_time = '11:00'
        self.conversion_api_base = 'http://currencies.apps.grandtrunk.net/getrate/%s/%s/%s'

    def get_attributes_dict(self, trip_obj):
        try:
            return trip_obj._attributes
        except AttributeError:
            return {}

    def str_to_int_float(self, str, default=0, to_int=True):
        if not str:
            return default
        try:
            if to_int:
                return int(str)
            return float(str)
        except (ValueError, TypeError):
            return default

    def room_bed_size(self, room_text):
        """
        Check the bed size in the room, from a list of
        predefined options.
        """
        bed_types = ['twin', 'queen', 'king', 'twinxl', 'full']
        for bed_type in bed_types:
            if bed_type.lower() in room_text.lower():
                return bed_type

    def parse_rooms(self, room_info):
        """
        Parse onsite purchase items for hotel from amenities
        """
        num_bed = ''
        bed_type = ''

        NUM_BED_PATTERN = re.compile(r'(\d\s+)', re.DOTALL)
        BED_TYPE_PATTERN = re.compile(r'(king|queen|double|twin|full)', flags=re.IGNORECASE | re.DOTALL)
        if room_info:
            # Search/match if number of bed is given
            num_bed_match = NUM_BED_PATTERN.search(room_info)
            if num_bed_match:
                num_bed = num_bed_match.group(1).strip()

            # Search/match if bed type is given
            bed_type_match = BED_TYPE_PATTERN.search(room_info)
            if bed_type_match:
                bed_type = bed_type_match.group(1).strip()

        return {'bed_number': self.str_to_int_float(num_bed) if num_bed else None,
                'bed_type': bed_type if bed_type else None}

    def calculate_airtime(self, departure, arrival):
        """
        Function to calculate flight airtime in seconds
        """
        if not (arrival and departure):
            return None

        try:
            time_difference = parser.parse(arrival) - parser.parse(departure)
            return time_difference.total_seconds()
        except ValueError:
            return 0

    def get_price_currency(self, amount_str):
        """
        Separate curreny and cost from amount string
        """
        amount_match = re.search(r'(\d+(,\d{3})*(\.?\d+))', amount_str)
        if not amount_match:
            return '', 0
        amount = amount_match.groups()[0]
        return amount_str.replace(amount, ''), int(self.str_to_int_float(amount.replace(',', ''), to_int=False) * 100)

    def is_smoking_allowed(self, room_spec_str):
        """
        Check if smoking is allowed in the room or not
        """
        return len(re.findall(r'non?\s*-?\s*smoking', room_spec_str, re.IGNORECASE)) == 0

    def extract_adult_child_count(self, raw_str):
        """
        Extract number of adults and number of children
        from the string. This returns the count in
        integer.
        """
        search_adult_child = re.search(r'(\d+)\s*adults?\s*,?\s*((\d+)\s*child)?',
                                       raw_str, re.IGNORECASE)
        if not search_adult_child:
            return 0, 0

        search_adult_child = search_adult_child.groups()
        adults = search_adult_child[0] if search_adult_child else 0
        childs = search_adult_child[2] if len(search_adult_child) > 2 else 0
        return self.str_to_int_float(adults), self.str_to_int_float(childs)

    def combine_date_and_time(self, date_or_string, time_string, elapsed_days=0):
        """
        Combine the provided date and time, return a
        datetime object
        """
        try:
            if isinstance(date_or_string, basestring):
                date_time_obj = parser.parse(date_or_string)
            else:
                date_time_obj = date_or_string
            if elapsed_days:
                date_time_obj += timedelta(days=elapsed_days)

            time = parser.parse(time_string)
            return date_time_obj.replace(hour=time.hour, minute=time.minute).isoformat()
        except ValueError:
            return ''

    def get_currency_converstion_rate(self, date, source_currency, base_currency='USD'):
        """
        This function returns the conversion rate from source
        to base currency at the provided date
        """
        if not (source_currency and date):
            return 0

        if isinstance(date, basestring):
            date = parser.parse(date)

        date = date.strftime('%Y-%m-%d')
        url = self.conversion_api_base % (date, source_currency, base_currency)
        response = requests.get(url)
        conversion_rate = self.str_to_int_float(response.content, to_int=False)
        return conversion_rate

    def timezone_to_seconds(self, raw_tz):
        raw_tz_list = raw_tz.replace('-', '').split(':')
        if len(raw_tz_list) < 2:
            return 0
        sec = self.str_to_int_float(raw_tz_list[0]) * 3600 + self.str_to_int_float(raw_tz_list[1]) * 60
        if '-' in raw_tz:
            return -sec
        return sec
