from __future__ import absolute_import

import logging
from datetime import timedelta

import re
from dateutil import parser

from lola.itinerary_parsing.parsers.api_parsers import LolaAPIParser
from lola.itinerary_parsing.utils import standardize_currency_name, get_tripit_email, tripit

logger = logging.getLogger('lola.api_parser')


class TripItApiParser(LolaAPIParser):
    def __init__(self, params):
        super(TripItApiParser, self).__init__()
        valid_token = tripit.OAuthConsumerCredential(params.get('consumer_key'),
                                                     params.get('consumer_secret'),
                                                     params.get('access_token'),
                                                     params.get('access_secret'))
        self.tripit_client = tripit.TripIt(valid_token)
        self.trip_ids = []
        self.provider = 'Tripit'
        self.email = get_tripit_email(params)
        self.annotations = []
        self.page_size = 50

    def get_trip_ids(self):
        logger.info('Fetching future trips')
        self.get_query_trip_ids({})
        logger.info('Fetching past trips')
        self.get_query_trip_ids({'past': 'true'})
        return self.trip_ids

    def parse_trip(self, trip_id):
        return self.get_trip_details(trip_id)

    def get_query_trip_ids(self, query):
        query['page_size'] = self.page_size
        page_num = 0
        max_page = 1

        while page_num < max_page:
            page_num += 1
            query['page_num'] = page_num
            trips = self.tripit_client.list_trip(query)
            trips_attrs = self.get_attributes_dict(trips)
            for trip_inst in trips._children:
                if trip_inst.__name__.lower() == 'trip':
                    self.trip_ids.append(trip_inst.id)

            page_num = int(trips_attrs.get('page_num', 0))
            max_page = int(trips_attrs.get('max_page', 0))
            if page_num == 1:
                logger.info("%s page(s) found with page size of %s" % (max_page, self.page_size))

    def get_trip_details(self, id):
        trip_obj = self.tripit_client.get_trip(id, {'include_objects': 'true'})
        itineraries = []
        for trip in trip_obj._children:
            if self.is_flight_object(trip):
                itineraries.append(self.parse_flight(trip, id))
            elif self.is_hotel_object(trip):
                itineraries.append(self.parse_hotel(trip, id))
        return itineraries

    def is_flight_object(self, obj):
        return 'airobject' in obj.__name__.lower()

    def is_hotel_object(self, obj):
        return 'lodging' in obj.__name__.lower()

    ########### FLIGHT ###########

    def parse_flight(self, flight_obj, trip_id):
        flight_attributes = self.get_attributes_dict(flight_obj)
        logger.info('Parsing flight trip of provider: %s' % flight_attributes.get('booking_site_name'))
        phone = flight_attributes.get('booking_site_phone')
        traveler_info = self.get_guest_info(flight_obj)
        cost_info = self.flight_cost_info(flight_attributes)
        segments = []
        for child in flight_obj._children:
            if child.__name__.lower() == 'segment':
                segments.append(self.parse_flight_segment(child))
        legs = self.create_legs_from_segments(segments)
        flight = {'legs': legs}
        flight.update(cost_info)

        itinerary = {
            'itinerary_id': '%s-%s' % (trip_id, flight_attributes.get('id', '')),
            'username': self.email,
            'cost_dollars': self.get_cost_dollars(cost_info,
                                                  self.get_flight_start_date(flight_attributes, segments),
                                                  is_flight=True),
            'source': self.provider,
            'transaction_id': flight_attributes.get('supplier_conf_num'),
            'hotel': {},
            'provider_name': flight_attributes.get('booking_site_name'),
            'provider_phone_number': phone,
            'flight': flight
        }
        itinerary.update(traveler_info)
        return itinerary

    def get_flight_start_date(self, attrbutes, segments):
        booking_date = attrbutes.get('booking_date')
        if booking_date:
            return booking_date
        if segments:
            return segments[0].get('departure')
        return None

    def parse_flight_segment(self, segment_obj):
        attrs = self.get_attributes_dict(segment_obj)
        flight_dates = self.parse_start_end_dates(segment_obj)
        segment = {
            'arrival_airport': attrs.get('end_airport_code'),
            'departure_airport': attrs.get('start_airport_code'),
            'cabin_type': attrs.get('service_class'),
            'cabin_type_string': attrs.get('service_class'),
            'marketing_airline': attrs.get('marketing_airline'),
            'flight_number': attrs.get('marketing_flight_number'),
            'seats': attrs.get('seats'),
            'arrival': flight_dates.get('end_date'),
            'departure': flight_dates.get('start_date'),
            'elapsed_time': self.get_segment_duration(flight_dates),
            'basis_code': None,
            'booking_code': None,
            'married_segment': None,
            'operating_airline': attrs.get('operating_airline'),
            'operating_airline_flight_number': attrs.get('operating_flight_number'),
            'stops': []
        }
        return segment

    def flight_cost_info(self, flight_attrs):
        base_curr, base_rate = self.get_price_currency(flight_attrs.get('booking_rate', ''))
        miles, cost_str = self.parse_miles_total_cost(flight_attrs.get('total_cost', ''))
        total_curr = ''
        total_rate = 0
        if cost_str:
            total_curr, total_rate = self.get_price_currency(cost_str)
        currency = standardize_currency_name(base_curr or total_curr)
        return {
            'currency_code': currency,
            'base_price': base_rate,
            'total_price': total_rate,
            'miles': miles
        }

    def parse_miles_total_cost(self, raw_str):
        raw_str = raw_str.replace(',', '')
        miles_cost = re.match(r'((\d+\.?\d+?)\s*Miles\s*\+\s*)?(.*)', raw_str, re.IGNORECASE)
        if not miles_cost:
            return [None, None]
        return [miles_cost.group(2), miles_cost.group(3)]

    def get_segment_duration(self, date_dict):
        if not (date_dict.get('start_date') and date_dict.get('end_date')):
            return 0
        try:
            start_date = parser.parse(date_dict.get('start_date')) - timedelta(seconds=date_dict.get('start_offset'))
            end_date = parser.parse(date_dict.get('end_date')) - timedelta(seconds=date_dict.get('end_offset'))
            return (end_date - start_date).total_seconds()
        except ValueError:
            return 0

    def create_legs_from_segments(self, segments):
        legs = []
        current_leg = {'segments': []}
        previous_airports = []
        for seg in segments:
            current_airports = [seg.get('arrival_airport'), seg.get('departure_airport')]
            if set(current_airports) == set(previous_airports):
                legs.append(self.get_leg_info(current_leg))
                current_leg = {'segments': []}
            previous_airports = current_airports
            current_leg['segments'].append(seg)
        legs.append(self.get_leg_info(current_leg))
        return legs

    def get_leg_info(self, leg):
        total_time = 0
        airports = []
        for seg in leg['segments']:
            total_time += seg.get('elapsed_time', 0)
            if seg['arrival_airport'] not in airports:
                airports.append(seg['arrival_airport'])

            if seg['departure_airport'] not in airports:
                airports.append(seg['departure_airport'])

        leg['total_time'] = total_time
        leg['num_segments'] = len(leg.get('segments', []))
        leg['airports'] = airports
        leg['interline_valid'] = ''
        leg['last_ticket_date'] = ''
        return leg

    ########### HOTEL ###########

    def parse_hotel(self, hotel_obj, trip_id):
        hotel_attributes = self.get_attributes_dict(hotel_obj)
        phone = hotel_attributes.get('booking_site_phone')
        logger.info('Parsing hotel trip of provider: %s' % hotel_attributes.get('booking_site_name'))
        price_info = self.get_hotel_cost_currency(hotel_attributes)
        address_info = self.get_hotel_address_details(hotel_obj)
        adult, child = self.extract_adult_child_count(hotel_attributes.get('number_guests', ''))
        guest_info = self.get_guest_info(hotel_obj)
        room_info = hotel_attributes.get('room_type', '')
        hotel_dates = self.parse_start_end_dates(hotel_obj)
        room_bed_info = self.parse_rooms(room_info)
        hotel = {
            'num_rooms': self.str_to_int_float(hotel_attributes.get('number_rooms')),
            'supplier_type': self.provider,
            'name': hotel_attributes.get('supplier_name'),
            'phone_number': phone,
            'smoking': self.is_smoking_allowed(room_info),
            'rooms': {
                'bed_type': room_bed_info
            },
            'checkin_date': hotel_dates.get('start_date'),
            'checkout_date': hotel_dates.get('end_date'),
            'currency_code': price_info.get('currency_code'),
            'base_rate': price_info.get('base_price'),
            'total_rate': price_info.get('total_price'),
            'high_rate': None,
            'low_rate': None,
            'tax_rate': None,
            'thumbnail_url': None,
            'trip_advisor_count': None,
            'trip_advisor_rating': None,
            'trip_advisor_url': None,
            'non_refundable': True,
            'onsite_purchases': None,
            'star_rating': None,
        }
        hotel.update(price_info)
        hotel.update(address_info)

        itinerary = {
            'itinerary_id': '%s-%s' % (trip_id, hotel_attributes.get('id', '')),
            'num_adults': adult,
            'num_children': child,
            'source': self.provider,
            'transaction_id': hotel_attributes.get('supplier_conf_num'),
            'flight': [],
            'provider_name': hotel_attributes.get('booking_site_name'),
            'provider_phone_number': phone,
            'provider_code': '',
            'hotel': hotel,
            'cost_dollars': self.get_cost_dollars(price_info, hotel_dates.get('start_date'))
        }
        itinerary.update(guest_info)
        return itinerary

    def get_hotel_cost_currency(self, trip_dict):
        base_curr, base_rate = self.get_price_currency(trip_dict.get('booking_rate', ''))
        total_cur, total_rate = self.get_price_currency(trip_dict.get('total_cost', ''))
        currency = standardize_currency_name(base_curr or total_cur)
        return {
            'currency_code': currency,
            'base_price': base_rate,
            'total_price': total_rate,
        }

    def get_hotel_address_details(self, hotel_obj):
        address = {}
        for child in hotel_obj._children:
            if child.__name__.lower() == 'address':
                address = self.get_attributes_dict(child)
                break
        return {
            'address1': address.get('address', ''),
            'city': address.get('city', ''),
            'province': address.get('state', ''),
            'postal_code': address.get('zip', ''),
            'latitude': address.get('latitude', ''),
            'longitude': address.get('longitude', ''),
        }

    ########### COMMON ###########

    def parse_start_end_dates(self, trip_obj):
        dates_info = {}
        for child in trip_obj._children:
            if 'startdate' in child.__name__.lower():
                attributes = self.get_attributes_dict(child)
                dates_info['start_date'] = self.combine_date_and_time(str(attributes.get('date', '')),
                                                                      str(attributes.get('time', '')))
                dates_info['start_offset'] = self.timezone_to_seconds(attributes.get('utc_offset', ''))
            if 'enddate' in child.__name__.lower():
                attributes = self.get_attributes_dict(child)
                dates_info['end_date'] = self.combine_date_and_time(str(attributes.get('date', '')),
                                                                    str(attributes.get('time', '')))
                dates_info['end_offset'] = self.timezone_to_seconds(attributes.get('utc_offset', ''))
        return dates_info

    def get_guest_info(self, hotel_obj):
        frequent_fliers = []
        travellers = []

        for child in hotel_obj._children:
            if child.__name__.lower() in ['guest', 'traveler']:
                attrs = self.get_attributes_dict(child)
                frequent_fliers.append(attrs.get('frequent_traveler_num', ''))
                travellers.append(attrs.get('first_name', ' ') + ' ' + attrs.get('last_name', ' '))

        return {
            'participants': [x.strip() for x in travellers if x.strip()],
            'rewards_id': ', '.join([x.strip() for x in frequent_fliers if x.strip()]),
            'rewards_program': 'frequent_traveler_num'
        }

    def get_cost_dollars(self, cost_dict, date, is_flight=False):
        currency = cost_dict.get('currency_code', '')
        if currency.lower() == 'usd':
            cr = 1
        else:
            try:
                cr = self.get_currency_converstion_rate(date, currency)
            except:
                cr = 0

        base = cost_dict.get('base_price', 0)
        total = cost_dict.get('total_price', 0)
        miles = cost_dict.get('miles', 0)
        cost_dollars = {
            'total': int(total * cr),
            'base': int(base * cr),
            'tax': 0,
            'fees': 0,
        }
        if is_flight:
            cost_dollars['miles'] = miles
        return cost_dollars
