# coding=utf-8
"""
Thi file contain google map api call to find google formatted cities and airport names
"""

from __future__ import absolute_import

import csv
import json
import logging
import os

import requests
from fuzzywuzzy import fuzz
from injector import singleton, inject
from requests.exceptions import ConnectionError
from lola.utils.config import LolaEnv
from redis import Redis

from lola.itinerary_parsing.parsers import LolaService


logger = logging.getLogger('lola.city')

# pylint: disable=no-self-use
@singleton
class CityLookupService(LolaService):
    """
    Class for lookup formatted city and airport names from google map api
    """

    @inject(redis=Redis, config=LolaEnv)
    def __init__(self, redis, config):
        super(CityLookupService, self).__init__()
        self.config = config
        self.redis = redis

    def parse_airport(self, airport_name):
        """
        Parse google formatted airport name from google map api
        """
        response = self.lookup_airport_on_google(airport_name)
        if not response:
            return None
        data = json.loads(response)
        results = data.get('results', None)
        name = None
        if not results:
            city = self.parse_city(airport_name)
            if city:
                name = city['name']
            return {'name': name}
        try:
            result = results[0]
            airport_name = result.get('name')
            return {'name': airport_name}
        except (LookupError, ConnectionError) as error:
            logger.exception('Exception in getting formatted airport name %s', error.message)
            return {'name': ''}

    def parse_city(self, city_name):
        """
        Parse google formatted city name from google map api
        """
        response = self.lookup_city_on_google(city_name)
        if not response:
            return None
        data = json.loads(response)
        results = data.get('results', None)
        if not results:
            return None
        try:
            result = results[0]
            name = None
            address_components = result['address_components']
            for component in address_components:
                if not name and component.get('short_name'):
                    name = component.get('short_name')
            return {'name': name}
        except LookupError, error:
            logger.exception('Exception in getting formatted airport name %s', error.message)
            return {'name': ''}


    def lookup_city_on_google(self, city_name):
        """
        Find google formatted city name using google map api
        """

        key = 'city_service:google:%s' % city_name
        if self.redis.exists(key):
            return self.redis.get(key)

        google_api_key = self.config.GOOGLE_API_KEY

        response = requests.get('https://maps.googleapis.com/maps/api/geocode/json', params={
            'address': city_name,
            'key': google_api_key
        })

        if response.status_code == 200:
            text = response.text
            self.redis.setex(key, text, 1209600)  # two weeks
            return text
        else:
            logger.info('Google map api error %s', response.status)
        return None

    def lookup_airport_on_google(self, airport_name):
        """
        Find google formatted airport name using google map api
        """

        key = 'airport_service:google:%s' % airport_name
        if self.redis.exists(key):
            return self.redis.get(key)

        google_api_key = self.config.GOOGLE_API_KEY

        response = requests.get('https://maps.googleapis.com/maps/api/place/textsearch/json', params={
            'query': airport_name,
            'key': google_api_key,
            'type': 'airport'
        })
        if response.status_code == 200:

            text = response.text
            self.redis.setex(key, text, 1209600)  # two weeks
            return text
        else:
            logger.info('Google map api error %s', response.status)
        return None

    def lookup_airport_code(self, airport_name, city_name):
        airport_codes = []
        csv.register_dialect('piper', delimiter='|', quoting=csv.QUOTE_NONE)
        script_dir = os.path.dirname(os.path.abspath(__file__))
        # Make airport information list (codes, city and airport name)
        with open(script_dir + '/updated_airport_names.txt', 'rb') as txtfile:
            for row in csv.DictReader(txtfile, dialect='piper'):
                airport_codes.append({'code': row['code'], 'city': row['city'], 'airport_name': row['airport_name']})
                # Match airport name and city name with fuzyy method and return airport code
        for airport_code in airport_codes:
            match_airport_code = fuzz.token_set_ratio(airport_name, airport_code['airport_name'])
            match_city_name = fuzz.token_set_ratio(city_name, airport_code['city'])
            # match airport name upto 90% and city name upto 100%
            if match_airport_code == 100 or (match_airport_code > 90 and match_city_name == 100):
                return airport_code['code']
        return None
