# -*- coding: UTF-8 -*-
from __future__ import absolute_import

import re

from bs4 import BeautifulSoup, NavigableString
from injector import inject

from lola.itinerary_parsing.parsers.city_lookup_service import CityLookupService
from lola.itinerary_parsing.parsers.hotel_lookup_service import HotelLookupService
from lola.itinerary_parsing.utils import is_empty_node, is_valid_node, is_table_header, convert_to_desired_val, \
    merge_child_nodes, parse_amount, standardize_currency_name, remove_empty_childs_airlines
from lola.itinerary_parsing.utils.currency_converter import CurrencyConverter
from lola.itinerary_parsing.parsers.email_parsers.rules.hotels import PATTERN_CURRENCY_AFTER, PATTERN_CURRENCY_BEFORE,\
    PATTERN_CURRENCY_SYMBOL, PATTERN_PURCHASE_AMOUNT_ONLY, VALIDATE_PATTERN_PURCHASE_AMOUNT_ONLY,\
    VALIDATE_PATTERN_PURCHASE_AMOUNT, PATTERN_PURCHASE_AMOUNT

TRAVEL_FILTERS = []

__all__ = ['generic', 'lookup_filter_class', 'TRAVEL_FILTERS']


def lookup_filter_class(source):
    for pattern in TRAVEL_FILTERS:
        if pattern['source'] == source:
            return pattern.get('filter', TravelFilter)
    return TravelFilter


def add_travel_filter(source, sender, subject, regex, filter_class):
    pattern = {'source': source, 'sender': sender, 'subject': subject, 'regex': regex, 'filter': filter_class}
    TRAVEL_FILTERS.append(pattern)


# define the decorator
def travel_filter(source, sender, subject, regex=None):
    def filter_decorator(cls):
        if subject:
            for subj in subject:
                add_travel_filter(source=source, sender=sender, subject=subj, regex=regex, filter_class=cls)
        else:
            add_travel_filter(source=source, sender=sender, subject=None, regex=regex, filter_class=cls)
        return cls

    return filter_decorator


class TravelFilter(object):
    @inject(city_service=CityLookupService, hotel_service=HotelLookupService)
    def __init__(self, city_service, hotel_service):
        self.city_service = city_service
        self.hotel_service = hotel_service
        pass

    def filter_source(self):
        return 'Generic'

    def parse_message(self, email):
        return {}

    def make_pattern(self, text):
        return re.compile(r'.*' + text, flags=re.DOTALL)

    def iterate_blobs(self, soup, pattern):
        blobs = self.matching_blobs(soup, self.make_pattern(pattern))
        for blob in blobs:
            yield blob

    def extract_contents_with_label(self, soup, pattern, **kwargs):
        for chunk in self.next_chunks(soup, pattern, **kwargs):
            # right now everything we need is in the first chunk; in the future
            # we will probably want to run our extraction across all chunks
            return chunk

        # if that fails, fall back on a simpler analysis that finds things like

        # Number of guests: 2

        regexp = pattern + r':\s*(.*)'

        matcher = re.compile(regexp)

        matches = matcher.findall(soup.getText(separator=u'\n'))
        for match in matches:
            return match

    # for working with HTML, at least one strategy is to search for "blobs" of text
    # which match certain patterns, then search outwards to find a region that contains
    # that.  I.e., search for "Rate Information" or "Room Type" or whatever, then search
    # recursively outwards to find self-contained chunks that are meaningful.  The idea
    # is that we iterate outwards but maybe only a couple steps, passing the chunks
    # to parsers that know what to look for.
    # An example of a strategy would be
    # * look for "Room Type"
    # * expand outwards to the containing table
    # * once in that table, look for patterns like "Rooms: <number of rooms>", "Clients: <number and type of clients>"
    #   Room Type: <type of room>
    # then within each of those patterns, do some extra parsing.

    def matching_blobs(self, html, pattern):
        soup = BeautifulSoup(html, "html5lib")
        tables = [table for table in soup.find_all('table') if pattern.match(table.getText(separator=u' '))]

        for table in reversed(tables):
            yield table
        pass

    def next_chunks(self, soup, pattern, multiline=False, include_anchors=False, parser=None, expected_type=None):
        """
        :param soup: a beautifulsoup node
        :param pattern: either a string or a regexp pattern
        :return: Text
        The basic idea of this function is that it finds an HTML element which matches the pattern, then finds the
        NEXT piece of content, through to the end of the table row (or whatever the containing element is)
        I.e., if the content is a table that looks like:
        Clients: 2 ADULTS
        Room Type: KING BED
        and you did get_next_chunk(..., 'Clients') then you would get back "2 ADULTS"
        """

        re_pattern = self.make_pattern(pattern)

        blobs = soup.findAll(text=re_pattern)

        # note that some of these blobs probably just have random crap in them.  we really want to know
        # if this pattern really represents a "label", which for now means that it is the last token
        # in its blob

        def is_anchor(blob):
            node = blob
            while node:
                if node.name == 'a':
                    return True
                else:
                    node = node.parent
            return False

        def is_label(blob, include_anchors=False):
            noise_patterns = ['date', 'Date', 'DATE', 'Summary']  # perhaps convert to lowercase?
            tokens = self.tokenize(unicode(blob))
            pattern_tokens = self.tokenize(pattern)
            while tokens[-1] in noise_patterns and tokens[-1] != pattern_tokens[-1]:
                tokens = tokens[0:-1]

            for i in range(1, len(pattern_tokens) + 1):
                if tokens[-i] != pattern_tokens[-i]:
                    return False

            # if the text matches, we still want to screen out text that is really just the anchor
            # text inside of an HTML link, which almost never is a valid label.
            if not include_anchors and is_anchor(blob):
                return False
            return True

        blobs = [blob for blob in blobs if is_label(blob, include_anchors=include_anchors)]

        def get_next_sibling_text(node, element_positions):
            """
            Method to collect text of previous sibling in HTML structure like e.g
            Best Available Rate |   Tax      |   Total for stay
            169.00 USD          |   19.77USD |   188.77USD
            In this type of structures Label and content are not present at sibling level, instead it is
            expected that label is a child of another element. This function tries to reach element which is
            a parent of label element so that we can categorize the purchase amount under given categories.
            BASE | TOTAL | TAX | FEE
            :return: Tuple. Containing category to which
            purchase amount belongs to and label text as a
            2nd element.
            """
            if node.next_sibling:
                sibling = node.next_sibling

                if not is_empty_node(sibling) and is_valid_node(sibling):
                    while element_positions:
                        sibling = sibling.contents[element_positions.pop()]
                    return sibling.text.strip(), False
                else:
                    return get_next_sibling_text(sibling, element_positions)
            else:
                element_positions.append(node.parent.index(node))
                return get_next_sibling_text(node.parent, element_positions)

        def collect_sib_text(node):

            def get_direct_text(node):
                collected_text = ''
                for item in node.descendants:
                    if isinstance(item, NavigableString) and item.parent.name != 'a':
                        collected_text = '%s %s' % (collected_text, item)
                    elif item.name == 'a':
                        continue
                return collected_text

            if not node:
                return None, False
            if node.next_sibling:
                # if there are nodes at this level, aggregate them
                t = ''
                # we really need to keep track of the font size, color, etc.
                # terminate the loop whenever there is a substantive change.
                # also, when we get to the end, after we've collected some text,
                # if we're in multiline mode then if we encounter a thing that
                # could be a label, or maybe a single line of an anchor, then
                # we're probably done.
                if is_table_header(node.parent):
                    element_position = [node.parent.index(node)]
                    return get_next_sibling_text(node.parent, element_position)
                for sib in node.next_siblings:
                    # short-circuit:
                    # if we have already collected some stuff so far we should stop
                    # if we hit a sibling which is an anchor.  We might also want
                    # to stop if we hit something that is a distinctly different typeface
                    # than what we've collected so far, but that is harder.
                    if t and not isinstance(sib, NavigableString) and sib.name == 'a':
                        # Collecting style attrs of child element exist under anchor element. If there exist hidden style we might need to continue
                        # collecting sibbling text.
                        style_child_sib = [child.attrs['style'] for child in sib.descendants if
                                           not isinstance(child, NavigableString) and 'style' in child.attrs]
                        if 'hidden' in ' '.join(style_child_sib):
                            continue
                        return t.strip(), False

                    # There might be a case when anchor tag exist as a child to sibling of element.
                    # Since getText returns text of all child without letting know from where it has
                    # collected text. Therefore func get_direc_text returns text of all elements excluding
                    # anchor element text
                    if t and not isinstance(sib, NavigableString) and bool(sib.find('a')):
                        t += get_direct_text(sib)
                        return t.strip(), False

                    # We need to stop collecting text if we have got h1 or image tag in siblings for label node
                    if t and (sib.name == 'h1' or sib.name == 'img'):
                        return t.strip(), False

                    try:
                        text = sib.getText(separator=u' ')
                    except Exception:
                        text = unicode(sib)

                    lines = text.splitlines()
                    if not lines:
                        lines = [text]
                    for line in lines:
                        content = line.strip()
                        if t:
                            t += ' '
                        t += content
                        if t and not multiline and ('\t' in line or len(lines) > 1):
                            return t.strip(), False
                    if t and not multiline and not isinstance(sib, NavigableString) and sib.name == 'br':
                        return t.strip(), False

                    # todo: another thing to think about here is "if we have already collected some text
                    # then maybe we want to throw away anchors and the like in here.
                    # this also probably applies to anchors that appear at the BEGINNING
                    # but I have no examples of those
                    if t and multiline and '\n' in text:
                        t += '\n'
                    if len(t) > 2 and t[-1] == '\n' and t[-2] == '\n':
                        return t.strip(), False
                try:
                    if expected_type:
                        collected_val = expected_type(t.strip())
                except ValueError:
                    return traverse_parents(node)
                return t.strip(), False

            # there could be this funky situation in which the 'label' quality here is really achieved
            # by a <span> or <font> or <b> tag that is inline followed by some other text.  So let's
            # look for that case specifically

            else:
                return traverse_parents(node)

        def traverse_parents(node):
            parent = node.parent
            text, parsed = collect_sib_text(parent)
            if text:
                if not parser:
                    return text, parsed
                else:
                    if parsed:
                        return text, parsed
                    data = parser(text)
                    if data:
                        return data, True
            return traverse_parents(parent)

        for blob in blobs:
            try:
                sib_text, parsed = collect_sib_text(blob)
                if parser and not parsed:
                    sib_text = parser(sib_text)
            except Exception, e:
                sib_text = None
            if sib_text:
                yield sib_text

    def tokenize(self, s, keepchars=''):
        # change params name/value into just the name/values
        s = re.sub('<param name=["\'](.+?)["\'] value=["\'](.+?)["\']', '\\1 \\2', s)
        # remove html tags
        s = re.sub('</?\w+', ' ', s)
        # remove any single s character used in RE
        s = re.sub('\s*s\?|\*|\+\s*', ' ', s)
        # now remove the rest and tokenize
        return re.sub(r'[^\w\-\_\.\~%%%s]+' % keepchars, ' ', s).strip().split(' ')

    def get_next_token(self, tokens, from_token):
        if from_token in tokens:
            index = tokens.index(from_token) + 1
            return tokens[index] if index < len(tokens) else None
        return None

    def get_tokens(self, tokens, matching):
        if isinstance(matching, (str, unicode)):
            matching = re.compile(matching)
        return [t for t in tokens if matching.search(t)]

    def combine_date_and_time(self, date_or_string, time_string):
        from dateutil.parser import parse
        if isinstance(date_or_string, basestring):
            dt = parse(date_or_string)
        else:
            dt = date_or_string
        if time_string:
           time = parse(time_string)
           return dt.replace(hour=time.hour, minute=time.minute)
        else:
            return dt

    def looks_like_itinerary_cost(self, dollars):
        # basically we want to ignore "large" round numbers such as those
        # in liability disclaimers.  We might do this eventually by looking
        # at the numbers in context.

        # Right now we dont use that logic
        # if dollars > 750:
        #     if dollars - int(dollars) == 0:
        #         return False
        return True

    def parse_record_locator(self, email, soup):
        patterns = {'record_locator': ['HotelTonight booking ID', 'Your confirmation number is', 'Your confirmation code is', 'Flight - Booking code', 'YOUR CONFIRMATION CODE',
                                       'Trip Confirmation Number', 'Your request number', 'Record Locator', 'Confirmation:', 'Confirmation', 'Confirmation Number',
                                       'Confirmation #', 'Confirmation Code', 'Confirmation number','Your confirmation number', 'Reservations',
                                       'CONFIRMATION', 'Booking reference', 'Booking Reference', 'Lufthansa booking code', 'Your reservation reference number',
                                       'AIR Confirmation', 'Lufthansa booking code:'
                                       ]}

        record_locator = None
        for key, triggers in patterns.iteritems():
            for trigger in triggers:
                chunk = self.extract_contents_with_label(soup, trigger, multiline=False)
                if chunk:
                    record_locator = chunk.strip()
                    break
        return record_locator

    def find_purchase_charges_breakup(self, email, soup, currency_code):

        """
        Functions finds Purchase amount break-up
        out of the email then tries to categorize
        purchase amount among different categories
        which are given below

        base rate | tax | fee | total
        :return: Tuple. Containing dictionary as a first element with key as
        category and purchase amount as a value against
        e,g {'base':200, 'fee':23: }
        and base, tax as a 2nd element and 3rd element
        """

        def get_currency_conversion_rate():
            """
            Get currency conversion rate
            """
            currency_convert = CurrencyConverter(currency_code)
            return currency_convert.get_conversion_rate(email.sent_at)

        def dig_down_element(element, positions):
            """
            Sometimes looking for category for purchase amount at sibling level is not sufficient,
            there might be case when a category can be found at sibling level of parent
            for current element. Therefore we need to track down of every position of
            element under parent while expanding our search for category or in other words
            while moving to parent where we can find siblings of parent.
            This method helps digging down sibling element from where we have started.
            """
            while positions:
                try:
                    element = element.contents[positions.pop()]
                    remove_empty_childs_airlines(element)
                    continue
                except (IndexError, AttributeError):
                    element = None
                    break
                positions.pop()
            return element

        def get_prev_sibling_text(node, element_position, element_pos_wth_br, element_positions):
            """
            Method to collect text of previous sibling in HTML structure like e.g
            Best Available Rate |   Tax      |   Total for stay
            169.00 USD          |   19.77USD |   188.77USD

            In this type of structures Label and content are not present at sibling level, instead it is
            expected that label is a child of another element. This function tries to reach element which is
            a parent of label element so that we can categorize the purchase amount under given categories.
            BASE | TOTAL | TAX | FEE
            :return: Tuple. Containing category to which
            purchase amount belongs to and label text as a
            2nd element.
            """
            remove_empty_childs_airlines(node.parent)
            if node.previous_sibling:
                for sibling in node.previous_siblings:
                    key_category = None
                    label_text = None

                    if not is_empty_node(sibling) and is_valid_node(sibling):
                        remove_empty_childs_airlines(sibling)

                        leaf_sibling = dig_down_element(sibling, element_positions[:])
                        if leaf_sibling:
                            key_category, label_text = self.categorize_purchase_amount(
                                leaf_sibling if isinstance(leaf_sibling, NavigableString) else leaf_sibling.text)
                        if key_category and label_text:
                            return key_category, label_text
                        # Since this function infers category of purchase amount from label text and label
                        # text is not present at sibling level rather then it is present as a child of any other
                        # node for example let me given you an example via html structure
                        #   <td>
                        #       Sub Total:<br>
                        #       *Estimated Tax and<br>
                        #       Other Charges:<br>
                        #       <b>Estimated Total:</b>
                        #   </td>
                        # <td >
                        #     $59.00<br>
                        #     $7.67<br><br>
                        #     <strong>$66.67</strong><br>(US Dollar)
                        # </td>
                        # Since child count of both the element are not same
                        # which won't allow you to make perfect mapping of purchase amount
                        # against it content label therefore merge_child_nodes function helps
                        # you to merge node into a single node by making an analysis of the node
                        # text, allowing you to make exact mapping of amount to label content.

                        # Estimated Tax and Other Charges is one label but according to
                        # structure of html beautifulsoup will consider it as a different
                        # children of node <td> making a difference in child count of td element
                        try:
                            sibling_labels = merge_child_nodes(sibling, element_pos_wth_br)
                            try:
                                key_category, label_text = self.categorize_purchase_amount \
                                    (sibling_labels[element_pos_wth_br])
                            except IndexError:
                                size = len(sibling.contents) - 1
                                if element_position <= size:
                                    child_node = sibling.contents[element_position]
                                else:
                                    child_node = sibling.contents[size]

                                child_node_text = child_node if isinstance(child_node, NavigableString) else child_node.text
                                validate_result = self.is_validate_label_text(child_node_text)
                                if validate_result:
                                    key_category, label_text = self.categorize_purchase_amount(child_node_text)

                        except AttributeError:
                            key_category, label_text = self.categorize_purchase_amount(
                                sibling if isinstance(sibling, NavigableString) else sibling.text)

                        if key_category and label_text:
                            return key_category, label_text
            else:
                if node.parent:
                    element_positions.append(node.parent.index(node))
                    return get_prev_sibling_text(node.parent, element_position,  element_position - len(node.find_previous_siblings('br'))
                                                         if node.find_previous_sibling('br')
                                                         else element_position, element_positions)
            return None, None

        def collect_prev_sib_text(node, element_positions):
            """
            Method to collect text of previous sibling in HTML structure like e.g
            Best Available Rate: 169.00 USD |    Tax: 19.77USD |   Total for stay: 188.77USD
            :return: Tuple, Category to which purchase amount belongs to
            and label text
            """
            if not node:
                return None, None
            remove_empty_childs_airlines(node.parent)
            if node.previous_sibling:
                try:
                    sibling_traverse_count = 0
                    element_position = node.parent.index(node)

                    for sib in node.previous_siblings:
                        leaf_sibling = dig_down_element(sib, element_positions[:])

                        if leaf_sibling:
                            key_category, label_text = self.categorize_purchase_amount(
                                leaf_sibling if isinstance(leaf_sibling, NavigableString) else leaf_sibling.text)
                            if key_category and label_text:
                                return key_category, label_text

                        # Following count keeps track of valid sibling parsed while
                        # finding category of purchase. If we won't find it till it reaches to
                        # current index of node under it's parent, then there might be different
                        # html structure to parse
                        sibling_traverse_count += 1
                        key, label_text = self.categorize_purchase_amount(sib if isinstance(sib, NavigableString)
                                                                          else sib.text)
                        if key and label_text:
                            return key, label_text

                        if sibling_traverse_count == element_position:
                            element_positions.append(node.parent.index(node))
                            return get_prev_sibling_text(node.parent, element_position,
                                                         element_position - len(node.find_previous_siblings('br'))
                                                         if node.find_previous_sibling('br')
                                                         else element_position, element_positions)

                    return collect_prev_sib_text(node.parent, element_positions)
                except AttributeError:
                    return None, None
            else:
                element_positions.append(node.parent.index(node))
                return collect_prev_sib_text(node.parent, element_positions)

        def prepare_label_amount(purchase_category, label_text, category_to_purchase_amount,
                                 amount):
            """
            Prepares dict which contains key as Purchase amount category and
            further dictionary values are again dict and this dict contains list
            as a value and items of these list are purchase values
            """
            amount = convert_to_desired_val(parse_amount(amount), float)
            if purchase_category in category_to_purchase_amount:
                if label_text in category_to_purchase_amount.get(purchase_category):
                    category_to_purchase_amount.get(purchase_category).get(label_text).append(amount)
                else:
                    category_to_purchase_amount.get(purchase_category).update({label_text: [amount]})
            else:
                category_to_purchase_amount.update({purchase_category: {label_text: [amount]}})

        def prepares_category_to_purchase(amounts):
            """
            Prepares dict containing purchase category as a key and under
            each category there is another dictionary which contains actual label
            text as key and further this dict contains list of purchase values
            :param List of purchase amount
            """
            category_to_purchase_amount = {}
            for amount in amounts:
                key, charges_breakup_label_text = self.categorize_purchase_amount(amount if
                                                                      isinstance(amount, NavigableString)
                                                                      else amount.text)
                if not key:
                    key, charges_breakup_label_text = collect_prev_sib_text(amount, [])
                if key:
                    prepare_label_amount(key, charges_breakup_label_text.strip(),
                                         category_to_purchase_amount, amount)
                else:
                    # if we are not able to categorize purchase amount try to categorize it
                    # by combining category keyword with amount and search it in email text
                    text = soup.getText(separator=u' ')
                    purchase_category = None
                    amount = amount.replace('$', r'\$').replace('(', r'\('). \
                        replace(')', r'\)')
                    charges_pattern = self.get_charges_pattern()
                    for key, patterns in charges_pattern.iteritems():
                        for pattern in patterns:
                            rule = re.compile(r'%s:(.*)%s' % (pattern, amount), re.IGNORECASE)
                            result = rule.search(text)
                            if result:
                                break
                        if result:
                            purchase_category = key
                            break
                    if purchase_category:
                        prepare_label_amount(purchase_category, result.group(0).strip(),
                                             category_to_purchase_amount, amount)
            return category_to_purchase_amount

        purchase_amounts = self.find_purchase_amount(soup)
        charges_keys_to_values_dict = {}
        if purchase_amounts:
            charges_keys_to_values_dict = prepares_category_to_purchase(purchase_amounts)

        # Summing up the values for Tax, Fee, excluding Total
        original_currency_charges = {'base': None, 'tax': None, 'fees': None, 'total': None}
        charges_after_conversion = {}
        if charges_keys_to_values_dict:
            charges_keys_to_values_dict.pop('exclude', None)
            original_currency_charges.update(self.sum_purchase_amounts(charges_keys_to_values_dict))
            charges_after_conversion.update(original_currency_charges)

            original_currency_charges.update(
                (x, convert_to_desired_val(round(y * 100))) for x, y in original_currency_charges.items() if
                original_currency_charges[x])

            conversion_rate = get_currency_conversion_rate()
            for key in charges_after_conversion.keys():
                if charges_after_conversion[key]:
                    charges_after_conversion[key] = convert_to_desired_val(
                        round(charges_after_conversion[key] * conversion_rate))

        return charges_after_conversion, original_currency_charges

    def find_purchase_amount(self, soup):
        """
        Finds all figure amount from email that
        looks like a purchase amount.
        :return List of purchase amount
        """
        purchase_amounts = []
        for pattern in [PATTERN_PURCHASE_AMOUNT, PATTERN_PURCHASE_AMOUNT_ONLY]:
            amounts = soup.findAll(text=pattern)
            if amounts:
                purchase_amounts.extend(amounts)
        return purchase_amounts

    def extract_currency_code(self, soup):
        """
        Function extracts currency code
        """
        text = unicode(soup.getText(separator=u' '))

        for currency_pattern in [PATTERN_CURRENCY_SYMBOL, PATTERN_CURRENCY_AFTER, PATTERN_CURRENCY_BEFORE]:
            match = currency_pattern.search(text)
            if match:
                currency_code = match.group(1)
                return standardize_currency_name(currency_code)

        return None

    def categorize_purchase_amount(self, sibling_text):
        """
        Categorize purchase amount.
        Finds category of purchase amount
        """
        # Since dict in python doesn't ensure order so added keys in
        # list to make sure we traverse dict as declared in list
        charge_category = self.get_charge_category_list()
        charges_pattern = self.get_charges_pattern()
        text = sibling_text.lower()
        for key in charge_category:
            patterns = charges_pattern[key]
            for pattern in patterns:
                if re.search(re.compile(r'\b\s*%s(s|es)?\s*\b' % pattern), text):
                    return key, text
        return None, None

    def sum_purchase_amounts(self, charges_keys_to_values_dict):
            return {}

    def get_charges_pattern(self):
        """
        Function to return charges pattern dic
        """
        return {}

    def get_charge_category_list(self):
        """
        Function to return charges category list
        """
        # There are purchase values which we don't want to include into our result
        # so exclude category will determine such values
        return ['exclude', 'total', 'tax', 'fees', 'base']

    def is_validate_label_text(self, label_text):
        """
        Function to validate purchase category label text,
        Sometimes due to bad structure we parse false category values. so we need to validate label by checking if there
         is not any amount in label text
        """
        # Amount in label indicate invalid label

        for pattern in [VALIDATE_PATTERN_PURCHASE_AMOUNT, VALIDATE_PATTERN_PURCHASE_AMOUNT_ONLY]:
            match_pattern = pattern.search(label_text)
            if match_pattern:
                return False
        return True