"""
This file contains the base class for airline
filters.
"""

from __future__ import absolute_import
import logging
from datetime import datetime, timedelta

from bs4 import BeautifulSoup, Comment
from dateutil.parser import parse

from lola.itinerary_parsing.utils import overrides, get_element_text
from lola.itinerary_parsing.parsers.email_parsers import TravelFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import CHARGE_PATTERN, ILLEGAL_TOKEN, \
    FORWARD_TRAVERSAL_LIMIT, BACK_TRAVERSAL_LIMIT
from lola.itinerary_parsing.utils import convert_to_str, convert_datetime_to_timestamp, \
    parse_email_information, calculate_airtime, strip_unicode, map_cabin_type, get_element_common_parent,\
    get_node_position_under_parent, remove_empty_childs_airlines, get_matched_text

__all__ = ['american', 'jetblue', 'delta', 'southwest', 'united', 'british', 'virginamerica',
           'alaska', 'spirit', 'aircanada', 'frontier', 'lufthansa', 'klm', 'airfrance', 'AirlineFilter']

logger = logging.getLogger('lola.airline')


# pylint: disable=no-self-use
class AirlineFilter(TravelFilter):
    """
    This is the Base class for airline emails parser.
    All airline parsers should be inherited from
    this.
    """

    def __init__(self):
        super(AirlineFilter, self).__init__()
        self.charge_pattern = CHARGE_PATTERN
        self.airline_rules = None
        self.flight_rules = None
        self.airline_rules_order = None
        self.segment_identify_rule = None
        self.date_format = None
        self.is_expansion_needed = None
        self.flight_segment_index = 0
        self.back_traversal_limit = BACK_TRAVERSAL_LIMIT
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html5lib'
        """
        return 'html5lib'

    def parse_message(self, email):
        """
        Parse email messages according to its filter class and then format flight data
        """
        text = email.html
        formatted_record = {}
        if not text or text == '':
            text = email.body
        text = text.replace('&nbsp;', ' ')
        soup = BeautifulSoup(strip_unicode(text), self.parser_class())
        # Parse flight email data from separate flight parsing class
        flights_data = self.parse_flights(email, soup)
        if flights_data:
            # Normalize flight destination names and get google formatted airport names from google place api
            self.normalize_destinations(flights_data)
            # format flight data according to required format
            formatted_record = self.canonicalize_flight_records(flights_data, email, soup)

        return formatted_record

    def canonicalize_flight_records(self, flights_data, email, soup):
        """
        Format flight records according to required Json format
        """
        logger.info("Parsing email for provider: %s", self.filter_source())
        properties = {}
        flights = flights_data.get('flights')
        flights_charges = flights_data.get('flights_charges')
        currency_code = self.extract_currency_code(soup)
        if not flights_charges:
            flights_charges, orignal_currency_charges = self.find_purchase_charges_breakup(email, soup, currency_code)

        if flights:
            transaction_id = flights[0].get('transaction_id')
            rewards_program = flights[0].get('rewards_program')
            rewards_id = flights[0].get('loyalty')
            if not transaction_id:
                record_locator = self.parse_record_locator(email, soup)
                if record_locator:
                    transaction_id = record_locator

            properties = {'user_id': email.sender,
                          'raw_email': parse_email_information(email),
                          'participants': self.augment_flight_participants(flights),
                          'num_children': None,
                          'num_adults': None,
                          'transaction_id': [transaction_id] if transaction_id else [],
                          'transaction_time': None,
                          'provider_name': self.filter_source(),
                          'provider_code': None,
                          'rewards_program': rewards_program,
                          'rewards_id': rewards_id,
                          'cost_dollars': flights_charges,
                          'flight': self.augment_flight_record(email, orignal_currency_charges, flights, currency_code),
                          'hotel': {}
                          }
        return properties

    def augment_flight_record(self, email, flights_charges, flights, currency_code):
        """
        Append flight charges to flight records legs
        """
        flights_records = {'base_price': flights_charges.get('base'),
                           'total_price': flights_charges.get('total'),
                           'tax': flights_charges.get('tax'),
                           'service_fee': flights_charges.get('fees'),
                           'currency_code': currency_code,
                           'legs': self.augment_flight_legs(email, flights)
                           }
        return flights_records

    def augment_flight_legs(self, email, flights):
        """
        Augment flight records into flight legs and flight stop
        """
        flights_legs = []
        leg_segment_list = []
        flight_airport_codes = []
        flights_size = len(flights) - 1
        for index, flight in enumerate(flights):
            # collect all flight segments/rows until we get complete flight leg
            leg_segment_list.append(flight)
            flight_airport_codes.append(flight.get('departure_airport_code'))
            flight_airport_codes.append(flight.get('arrival_airport_code'))

            if flights_size == index:
                flights_legs.append(
                    self.augment_flight_legs_segments(email, leg_segment_list, flight_airport_codes))
                leg_segment_list = []
                flight_airport_codes = []
            else:
                # we are comparing departure city of first index and arrival city of next index
                # If both city are equal that indicate the complete one leg of flight
                try:
                    if flight.get('departure_city').strip() == flights[index + 1].get('arrival_city').strip():
                        flights_legs.append(
                            self.augment_flight_legs_segments(email, leg_segment_list, flight_airport_codes))
                        leg_segment_list = []
                        flight_airport_codes = []
                except (LookupError, AttributeError):
                    pass

        return flights_legs

    def augment_flight_legs_segments(self, email, leg_segment_list, flight_airport_codes):
        """
        Augment flight legs segments
        """
        segments = []
        current_segment = {}
        total_time = 0
        for leg_segment in leg_segment_list:
            # We need total air time from each flight segment
            airtime = calculate_airtime(leg_segment.get('departure'), leg_segment.get('arrival'))
            total_time = total_time + airtime
            if leg_segment.get('flight_number') and current_segment:
                if current_segment.get('flight_number') == leg_segment.get('flight_number'):
                    stops = {'airport': leg_segment.get('departure_airport'),
                             'arrival': convert_datetime_to_timestamp(leg_segment.get('arrival')),
                             'departure': convert_datetime_to_timestamp(leg_segment.get('departure')),
                             'duration': airtime,
                             'plane_type': leg_segment.get('plane_type')
                    }
                    current_segment['stops'].append(stops)
                    current_segment['arrival_airport'] = leg_segment.get('arrival_airport')
                    current_segment['arrival'] = convert_datetime_to_timestamp(leg_segment.get('arrival'))
                    try:
                        elapsed_time = calculate_airtime(parse(current_segment.get('departure')),
                                                         leg_segment.get('arrival'))
                    except (ValueError, AttributeError):
                        elapsed_time = 0
                        pass
                    current_segment['elapsed_time'] = elapsed_time
                    continue
                else:
                    segments.append(current_segment)
            current_segment = {
                'plane_type': leg_segment.get('plane_type'),
                'departure_airport': leg_segment.get('departure_airport'),
                'arrival_airport': leg_segment.get('arrival_airport'),
                'arrival': convert_datetime_to_timestamp(leg_segment.get('arrival')),
                'basis_code': None,
                'booking_code': None,
                'cabin_type': map_cabin_type(leg_segment.get('seat_class')) if leg_segment.get('seat_class') else None,
                'cabin_type_string': leg_segment.get('seat_class') if leg_segment.get('seat_class') else None,
                'departure': convert_datetime_to_timestamp(leg_segment.get('departure')),
                'elapsed_time': airtime,
                'flight_number': convert_to_str(leg_segment.get('flight_number')) if leg_segment.get(
                    'flight_number') else None,
                'marketing_airline': leg_segment.get('marketing_airline').strip() if leg_segment.get(
                    'marketing_airline') else self.filter_source(),
                'married_segment': None,
                'operating_airline': leg_segment.get('operating_airline').strip() if leg_segment.get(
                    'operating_airline') else self.filter_source(),
                'operating_airline_flight_number': convert_to_str(
                    leg_segment.get('operating_airline_flight_number')) if leg_segment.get(
                    'operating_airline_flight_number') else None,
                'stops': []
            }

        segments.append(current_segment)
        legs = {'interline_valid': None,
                'last_ticket_date': convert_datetime_to_timestamp(str(email.sent_at)) if email.sent_at else None,
                'num_segments': len(segments),
                'total_time': int(total_time),
                'airports': list(set(flight_airport_codes)),
                'segments': segments}
        return legs

    def augment_flight_participants(self, flights):
        """
        Format flight participants into required Json format
        """
        # Some time we get duplicate participant name so make a unique list of flight participant
        participants = []
        for flight in flights:
            name = flight.get('name')
            if name:
                # Multiple names are join with separator '|'
                name = name.split('|')
                for name_node in name:
                    participants.append(name_node)
        return list(set(participants))

    def process_segment_identify_elements(self, segment_rule, soup):
        segment_identify_elements = []
        rule = None
        for pattern in segment_rule:
            segment_pattern = pattern.get('rule', '')
            segment_identify_elements = soup.findAll(text=segment_pattern)

            if pattern.get('is_validate'):
                segment_identify_elements = self.filter_starting_segment_elements(segment_identify_elements[:],
                                                                                  segment_pattern)
            segment_identify_elements = self.make_pair_starting_elements(segment_identify_elements[:],
                                                                         segment_pattern,
                                                                         pattern.get('is_shift'))
            if segment_identify_elements:
                rule = pattern
                break
        return segment_identify_elements, rule


    def parse_flights(self, email, soup):
        """
        Parse flight emails for generic airlines
        """
        flights = []
        for segment_rule in self.segment_identify_rule:
            segment_identify_elements, rule = self.process_segment_identify_elements(segment_rule, soup)
            if not segment_identify_elements:
                continue

            # Intention of getting common parent is to avoid badly structured html.
            # We are getting common parent of starting and last element in segment identify list so that
            # we can reach a level where we can pick values of flight from siblings.
            # Getting common parent help us to reach at a level where segment identify elements gets at sibling
            # level.

            comparison_text = segment_identify_elements[-1]
            is_single_segment = len(segment_identify_elements) <= 2

            if is_single_segment:
                segment_identify_elements[0] = self.enhance_element_level(segment_identify_elements[0])

            common_parent = get_element_common_parent(segment_identify_elements[0],
                                                      comparison_text, is_single_segment)

            index_to_segment = {}
            common_parent = common_parent.parent if len(segment_identify_elements) == 2 else common_parent
            flight_segments = self.divide_segments(segment_identify_elements, common_parent)

            # Once we are at sibling level of elements now get position/index of an element under it's parent.
            # This will help us to mark segments boundaries to begin our search
            # for flight information. This way we can avoid conflict between segments.
            for segment in flight_segments:
                element_pos, _ = get_node_position_under_parent(
                    common_parent, segment)
                if element_pos >= 0:
                    index_to_segment[element_pos] = segment

            flight_section_keys = index_to_segment.keys()
            flight_section_keys.sort()

            segment_rules = rule.get('segment_rule')
            segment_back_track_count = 0

            for index, val in enumerate(flight_section_keys):
                # Checking index value if it is greater than zero than assigning previous index with negative
                # of back traversal limit so that code can make a back traversal for initial element. Since code
                # checks for difference between previous index and current index in order to determine space for back
                # traversal.
                prev_element_index = flight_section_keys[index - 1] if index else -self.back_traversal_limit if \
                    flight_section_keys[index] <= 1 else 0

                flight_information = {}
                self.airline_rules = segment_rules
                self.flight_rules = segment_rules.copy()
                self.is_expansion_needed = segment_rules.get('is_expansion_needed')
                self.flight_segment_index = val

                flight_information, segment_back_track_count = self.extract_airline_info(
                    flight_segment=index_to_segment[val], flight_information=flight_information, curr_elem_index=val,
                    next_elem_index=val + self.forward_traversal_limit if index + 1 > len(
                        flight_section_keys) - 1 else flight_section_keys[index + 1],
                    backward_traverse_limit=segment_back_track_count, prev_elem_index=prev_element_index,
                    is_single=len(flight_segments) == 1)

                flights.append(flight_information)

        flights = self.merge_segments(flights)
        for flight in flights:
            self.enhance_flight_segment(flight, email, soup)

        return {'flights': flights}

    def normalize_destinations(self, flight_data):
        """
        Get google formatted airport name from google place api and then match airport codes from airport code file
        """
        flights = flight_data.get('flights', [])
        for flight in flights:
            departure_airport_code = flight.get('departure_airport_code')
            arrival_airport_code = flight.get('arrival_airport_code')
            departure_city = flight.get('departure_city')
            arrival_city = flight.get('arrival_city')

            if not departure_airport_code and departure_city:
                departure_airport_code = self.get_airport_code(departure_city)
            if not arrival_airport_code and arrival_city:
                arrival_airport_code = self.get_airport_code(arrival_city)

            flight['departure_airport'] = departure_airport_code
            flight['departure_airport_code'] = departure_airport_code
            flight['arrival_airport'] = arrival_airport_code
            flight['arrival_airport_code'] = arrival_airport_code

    def get_airport_code(self, city):
        """
        Get google formatted city/airport names and then match airport code
        """
        airport_code = None
        parsed_city = self.city_service.parse_airport(city)
        if parsed_city:
            airport_code = self.city_service.lookup_airport_code(parsed_city.get('name'), city)
        return airport_code

    @overrides(TravelFilter)
    def sum_purchase_amounts(self, charges_keys_to_values_dict):
        purchase_charges = {}
        for category, charges_info in charges_keys_to_values_dict.iteritems():
            amount_charges = []
            for charges in charges_info.values():
                amount_charges.extend(charges)
            try:
                if category == 'total':
                    purchase_charges[category] = max(amount_charges)
                    continue
                purchase_charges[category] = sum(amount_charges)
            except TypeError:
                pass
        # if we did not get total amount then sum all charges
        if not purchase_charges.get('total'):
            purchase_charges['total'] = sum(purchase_charges.values())
        return purchase_charges

    @overrides(TravelFilter)
    def get_charges_pattern(self):
        """
        Function to return charges pattern dic
        """
        return self.charge_pattern

    def divide_segments(self, starting_elements, common_parent):
        """
        This function divides segments on the behalf of segment identifying elements,
        most of the time we prefer to divide segments on time element, because most
        of the time it is expected there must be a time or airport name against any segment.
        """
        if not common_parent or not starting_elements:
            return []

        flight_contents = []
        length = len(starting_elements)

        for index in range(0, length, 2):
            flight_content = self.get_flight_content_target(starting_elements[index], common_parent)
            if flight_content:
                flight_contents.append(flight_content)

        if flight_contents:
            remove_empty_childs_airlines(flight_contents[0].parent)
        return flight_contents

    def make_pair_starting_elements(self, starting_elements, pattern, is_shift_array):
        """
        Since segments are divided on the behalf of the elements belongs to different segments.
        We have to make sure that we wont divide segment on the behalf of the element which belongs
        to same segment. Therefore this function make sure elements that don't belongs to same segments
        must not lie consecutively in list.
        Furthermore this function removes any unwanted elements from the list.
        """
        filtered_elements = []
        if is_shift_array:
            filtered_elements = [elem for elem in starting_elements if not isinstance(elem, Comment) for _ in
                                 range(0, 2)]
            return filtered_elements

        while starting_elements:
            for index, elem in enumerate(starting_elements):
                if isinstance(elem, Comment):
                    starting_elements.pop(index)
                    break

                filtered_elements.append(elem)
                res = pattern.findall(elem)
                if res:
                    length = len(res) - 1
                    for _ in range(0, length):
                        filtered_elements.append(elem)
                starting_elements.pop(index)
                break
        return filtered_elements

    def filter_starting_segment_elements(self, starting_elements, pattern):
        """
        It is necessary for us to filter out elements that does not belongs
        to same segment. This function filters out element by picking 2
        consecutive element from the list and then gets common parent for the
        elements and then checks, that there should be 2 element of the same
        pattern which these element have, under the common parent.
        This way we can avoid getting wrong values for segment identifying elements.
        This function is not applicable for all the pattern its usability depends on html
        structure.
        """
        filtered_elements = []

        while starting_elements:
            for count in range(0, len(starting_elements), 2):
                if isinstance(starting_elements[count], Comment):
                    starting_elements.pop(count)
                    break

                try:
                    elem1 = starting_elements[count]
                    elem2 = starting_elements[count + 1]
                except IndexError:
                    return filtered_elements

                flight_section = get_element_common_parent(elem1, get_element_text(elem2))

                if not self.is_valid_segment_element(flight_section, pattern):
                    starting_elements.pop(count)
                    break
                filtered_elements.append(elem1)
                filtered_elements.append(elem2)
                starting_elements.remove(elem1)
                starting_elements.remove(elem2)
                break
        return filtered_elements

    def extract_airline_info(self, flight_segment, flight_information, curr_elem_index=0,
                             next_elem_index=0, back_traverse_count=0, backward_traverse_limit=0,
                             is_backward=False, prev_elem_index=0, is_single=False):
        """
        Function marks the boundary of the section using rules for airlines.
        Purpose of marking boundary for a section is to avoid picking values of
        airlines from wrong segment.
        This function makes traversal in backward and forward direction looking
        for the values which belongs to a particular segment.
        """
        text = get_element_text(flight_segment, '\n')
        text = self.replace_text(text, ILLEGAL_TOKEN)
        match = None

        # Following part of code helps identify how much forwards traversal
        # is required for finding values for flight segment.
        # This help us not exceed our search to next flight segment.

        # curr_elem_index: keep track of traversal count while finding flight values
        # from current segment.
        # next_elem_index: This variable help knowing when next sections starts.
        # backward_traverse_limit: Keeps track of back traversal of previous segment.
        if (next_elem_index - backward_traverse_limit) - curr_elem_index <= 0 \
                and self.flight_segment_index != curr_elem_index:

            if self.is_expansion_needed and self.is_expansion_needed(flight_information, is_single):
                flight_segment = self.enhance_element_level(flight_segment)
                if is_single:
                    return self.expand_search(flight_segment, is_single)
            # Before switching to next segment updating value of back traversal count
            self.back_traversal_limit = backward_traverse_limit
            return flight_information, backward_traverse_limit

        for rule_name in self.airline_rules_order:
            rule = self.flight_rules.get(rule_name, None)
            if not rule:
                continue

            rule_pattern = None
            is_replace = True
            is_match_again = None
            match = None
            if isinstance(rule, list):
                for item in rule:
                    rule_pattern = item.get('rule')
                    is_replace = item.get('is_replace')
                    is_match_again = item.get('match_again')

                    if not isinstance(rule_pattern, str):
                        match = rule_pattern.findall(text)
                    if match:
                        rule = item
                        break
            else:
                rule_pattern = rule.get('rule')
                is_replace = rule.get('is_replace')
                is_match_again = rule.get('match_again')
                match = rule_pattern.findall(text)

            if match:
                if prev_elem_index <= 0 and is_backward:
                    backward_traverse_limit += 1
                if is_replace:
                    text = self.replace_text(text, match)
                matched_text = get_matched_text(match)

                if is_match_again:
                    match = rule_pattern.findall(text)
                    while match:
                        matched_text.extend(get_matched_text(match))
                        text = self.replace_text(text, match)
                        if text.isspace() or not bool(text.strip()):
                            break
                        match = rule_pattern.findall(text)

                if not rule['is_repeated']:
                    self.flight_rules.pop(rule_name, None)

                if rule_name in flight_information:
                    flight_information[rule_name].extend(matched_text)
                else:
                    flight_information[rule_name] = matched_text

        if back_traverse_count < self.back_traversal_limit and flight_segment.previous_sibling:
            back_traverse_count += 1
            # Making sure if current and previous element has some space between them.
            if curr_elem_index - prev_elem_index > 1:
                backward_traverse_limit, back_traverse_count = self.extract_airline_info(
                    flight_segment=flight_segment.previous_sibling, flight_information=flight_information,
                    curr_elem_index=curr_elem_index, next_elem_index=next_elem_index,
                    back_traverse_count=back_traverse_count,
                    backward_traverse_limit=backward_traverse_limit, is_backward=True, prev_elem_index=prev_elem_index)

        # Following variable help identifies backward move for looking segment values.
        if is_backward:
            # Since this function make move on both direction forward and backward
            # in search for the desired field values, therefore this function returns
            # different values for different directions. Backward direction return
            # back traversal limit, back traverse count and Forwards returns
            # dictionary holding field values, back_traversal_limit
            return backward_traverse_limit, self.back_traversal_limit \
                if back_traverse_count < self.back_traversal_limit else back_traverse_count

        curr_elem_index += 1
        if flight_segment.next_sibling:
            return self.extract_airline_info(flight_segment=flight_segment.next_sibling,
                                             flight_information=flight_information,
                                             curr_elem_index=curr_elem_index, next_elem_index=next_elem_index,
                                             prev_elem_index=prev_elem_index,
                                             back_traverse_count=back_traverse_count,
                                             backward_traverse_limit=backward_traverse_limit, is_single=is_single)
        else:
            if self.is_expansion_needed and self.is_expansion_needed(flight_information, is_single):
                flight_segment = self.enhance_element_level(flight_segment)

                if is_single:
                    return self.expand_search(flight_segment, is_single)
            return flight_information, backward_traverse_limit

    def expand_search(self, flight_segment, is_single):
        """
        Sometimes we are not on a level where we can collect value for all
        properties. Therefore this function helps expanding search to reach level
        where we can get all values.
        """
        if not flight_segment.parent:
            return {}, 0

        flight_information = {}
        self.flight_rules = self.airline_rules.copy()
        parent = flight_segment.parent
        remove_empty_childs_airlines(parent)
        index, _ = get_node_position_under_parent(parent, flight_segment)
        return self.extract_airline_info(flight_segment=flight_segment,
                                         flight_information=flight_information, curr_elem_index=index,
                                         next_elem_index=index + self.forward_traversal_limit,
                                         prev_elem_index=index - self.back_traversal_limit,
                                         is_single=is_single)

    def is_valid_segment_element(self, section, pattern):
        """
        Checks if there exist of 2 elements of the pattern under
        an element.
        """
        return bool(len(section.findAll(text=pattern)) == 2)

    def replace_text(self, text, replace_text):
        """
        This function removes matched text from segment text.
        So that removed text might not conflict with another search
        on the same text.
        """
        for item in replace_text:
            if isinstance(item, basestring):
                text = text.replace(item, '')
                continue
            for replace_item in item:
                text = text.replace(replace_item, '')
        return text

    def enhance_flight_segment(self, segments, email, soup):
        """
        This function receives result dictionary and convert raw values to
        specific format.
        """
        departing_date, departing_time, arrival_date, arrival_time = (None,) * 4

        for field_name in self.airline_rules_order:
            try:
                self.enhance_flight_info(segments, field_name, segments.get(field_name, None), soup)
                value = segments.get(field_name, None)
                if not value:
                    continue

                value = self.clean_string(value)
                if field_name == 'airport':
                    try:
                        val1, val2 = value[0], value[1]
                        segments.update({'departure_city': val1, 'arrival_city': val2})
                        segments.pop(field_name)
                    except IndexError:
                        segments.update({'departure_city': None, 'arrival_city': None})
                elif field_name == 'time':
                    val1, val2 = (None,) * 2
                    try:
                        val1, val2 = value
                    except ValueError:
                        val1 = value[0]
                    departing_time, arrival_time = val1, val2
                    segments.pop(field_name, None)
                elif field_name == 'name':
                    segments.update({field_name: '|'.join(value)})
                    continue
                elif field_name == 'flight_date':
                    date_values = []
                    for item in self.date_format:
                        for val in value:
                            try:
                                val = val.replace('.', '')
                                date_values.append(datetime.strptime(val, item))
                            except ValueError:
                                pass

                    for val in date_values:
                        if departing_date:
                            if departing_date > val:
                                arrival_date = departing_date
                            else:
                                arrival_date = val
                                break
                        departing_date = val

                    if not departing_date or not arrival_date:
                        departing_date = date_values[0]
                        arrival_date = date_values[0]

                    segments.update({'departure': departing_date, 'arrival': arrival_date})
                    segments.pop('flight_date')
                elif field_name == 'airport_code':
                    try:
                        segments.update({'departure_airport_code': value[0], 'arrival_airport_code': value[1]})
                    except IndexError:
                        segments.update({'departure_airport_code': None, 'arrival_airport_code': None})
                    segments.pop(field_name)
                else:
                    value = list(set(value))
                    segments.update({field_name: '|'.join(value)})
            except (ValueError, TypeError, IndexError):
                pass

        try:
            if departing_date:
                departing_date = self.combine_date_and_time(departing_date, departing_time)
                if departing_date.year == 1900:
                    departing_date = departing_date.replace(year=email.sent_at.year)

            if arrival_date:
                arrival_date = self.combine_date_and_time(arrival_date, arrival_time)
                if arrival_date.year == 1900:
                    arrival_date = arrival_date.replace(year=email.sent_at.year)
        except ValueError:
            pass

        if arrival_date and departing_date and (arrival_date < departing_date):
            arrival_date = arrival_date + timedelta(days=1)
        segments.update({'departure': departing_date, 'arrival': arrival_date})

    def get_flight_content_target(self, target, parent):
        """
        Get flight content target by match two target texts
        """
        while target.parent:
            if target.parent != parent:
                target = target.parent
                continue
            break
        return target

    def merge_segments(self, flights):
        """
        Sometimes properties like 'name', 'FF' etc is not found within the flight
        segment. This function scans through the flight segments list and look if any of the
        information other than flights exists which requires a merger with flight segments.
        """
        flight_segments = []
        other_segments = []
        for index, flight in enumerate(flights):
            if 'airport' in flight or 'airport_code' in flight:
                if 'flight_date' not in flight and index >= 1:
                    flight.update({'flight_date': flights[index - 1].get('flight_date')})
                flight_segments.append(flight)
                continue
            other_segments.append(flight)

        if other_segments:
            for flight in flight_segments:
                for other in other_segments:
                    common_properties = set(flight).intersection(set(other))
                    for prop in common_properties:
                        try:
                            flight[prop].extend(other.pop(prop))
                        except TypeError:
                            pass
                    flight.update(other)
        return flight_segments

    def enhance_flight_info(self, segments, flight_info, value, soup):
        """
        There are some properties which couldn't be picked by generic class.
        So in this scenario child class override this function to add
        property value in result dictionary.
        """
        pass

    def clean_string(self, values):
        """
        Removes extra spaces
        """
        if values:
            values = [' '.join(val.replace('\n', '').split()) for val in values if val and val.strip()]
            return values
        return ''

    def enhance_element_level(self, element):
        """
        In case of single segment this function tries to traverse to parent of the
        element until it find sibling to the parent.
        """
        try:
            remove_empty_childs_airlines(element.parent)
            if element:
                while not element.next_sibling and element.parent:
                    element = element.parent
                    remove_empty_childs_airlines(element.parent)
        except (AttributeError, TypeError):
            pass
        return element
