"""
The file contains the parsing/overrides functions for British Airways emails.
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.british import PASSENGER_NAME_REGEX, PASSENGER_TARGET_REGEX, \
    BRITISH_RULE, BRITISH_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES
from lola.itinerary_parsing.utils import find_target, get_next_sibling_target, get_node_contents, get_matched_text

logger = logging.getLogger('lola.airline')


@travel_filter(source='British Airways', sender='ba@britishairways.com',
               subject=['BA check-in confirmation', 'Your e-ticket receipt', 'BA e-ticket receipt'])
@travel_filter(source='British Airways', sender='ankitdesai88@gmail.com',
               subject=['BA check-in confirmation', 'Your e-ticket receipt', 'BA e-ticket receipt'])
# pylint: disable=no-self-use
class BritishFilter(AirlineFilter):
    """
     Base class for British Airways emails parsing.
    Override functions for British Airways are in this class
    """

    def __init__(self):
        super(BritishFilter, self).__init__()
        self.airline_rules = BRITISH_RULE
        self.airline_rules_order = BRITISH_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently 'British Airways' is returning
        """
        return 'British Airways'

    @overrides(AirlineFilter)
    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html.parser'
        """
        return 'html.parser'

    @overrides(AirlineFilter)
    def enhance_flight_info(self, segments, flight_info, value, soup):
        if flight_info != 'name':
            return
        passenger_names = self.parse_flight_passengers(soup)
        flight_dict = {'name': []}
        if passenger_names:
            flight_dict.get('name').extend(passenger_names)
        segments.update(flight_dict)

    def parse_flight_passengers(self, soup):
        """
        Parse flight passengers names with regex if we get don't get passenger names in target
        then find its next siblings targets
        """
        passenger_list = []
        passenger_target = find_target(soup, PASSENGER_TARGET_REGEX)

        while passenger_target:
            for node_contents in get_node_contents(passenger_target):
                match = PASSENGER_NAME_REGEX.findall(node_contents)
                if match:
                    passenger_list.extend(get_matched_text(match))
            if not passenger_list:
                passenger_target = get_next_sibling_target(passenger_target, is_parent=True)
                continue
            break
        return passenger_list

    @overrides(AirlineFilter)
    def sum_purchase_amounts(self, charges_keys_to_values_dict):
        # We are getting a fee value in total category so shift that value to fees category
        for label, charges in charges_keys_to_values_dict['total'].iteritems():
            # seat charges will come under fees category
            if 'seat' in label:
                charges_keys_to_values_dict['fees'].update({'seat': charges})
        return super(BritishFilter, self).sum_purchase_amounts(charges_keys_to_values_dict)
