"""
The file contains the parsing functions for Delta airline emails.
We are parsing flight information like flight legs and flight charges
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
import logging
import re

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.delta import TRAVEL_FILTER_REGEX, \
    DELTA_REGEX, DELTA_RULES_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES

logger = logging.getLogger('lola.airline')


@travel_filter(source='Delta Airlines', sender='rg.vanessa@gmail.com', subject=None, regex=[TRAVEL_FILTER_REGEX])
@travel_filter(source='Delta Airlines', sender='DeltaAirLines@e.delta.com', subject=None, regex=[TRAVEL_FILTER_REGEX])
@travel_filter(source='Delta Airlines', sender='omair.shamshir@arbisoft.com', subject=None, regex=[TRAVEL_FILTER_REGEX])
# pylint: disable=no-self-use
class DeltaFilter(AirlineFilter):
    """
    Base class for Delta airline emails parsing.
    Override functions for delta airline are in this class
    """

    def __init__(self):
        super(DeltaFilter, self).__init__()
        self.airline_rules = DELTA_REGEX
        self.airline_rules_order = DELTA_RULES_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline 'Delta Airlines'
        """
        return 'Delta Airlines'

    @overrides(AirlineFilter)
    def enhance_flight_info(self, segments, flight_info, value, soup):
        if not value or flight_info != 'flight_number':
            return

        digit_pattern = re.compile(r'\d+')
        value = [digit_pattern.search(val).group(0) for val in value]
        segments.update({flight_info: value})

        def update_flight_info(segments, rule, key):
            result = None
            segment_rule = rule.get('segment_rule').get(key)
            if segment_rule:
                if isinstance(segment_rule, list):
                    for item in segment_rule:
                        if isinstance(item.get('rule'), str):
                            result = re.search(
                                re.compile(item.get('rule') % value[0]), soup.text)
                elif isinstance(segment_rule.get('rule'), str):
                    result = re.search(
                        re.compile(segment_rule.get('rule') % value[0]),
                        soup.text)

            if result:
                segments.update({key: result.group(1)})

        for rules in self.segment_identify_rule:
            for rule in rules:
                if rule.get('is_validate'):
                    update_flight_info(segments, rule, 'operating_airline')
                    update_flight_info(segments, rule, 'operating_airline_flight_number')
        return

    @overrides(AirlineFilter)
    def sum_purchase_amounts(self, charges_keys_to_values_dict):
        purchase_charges = {}
        for category, charges_info in charges_keys_to_values_dict.iteritems():
            amount_charges = []
            for charges in charges_info.values():
                amount_charges.extend(charges)
            try:
                if category == 'total':
                    purchase_charges[category] = max(amount_charges)
                    continue
                purchase_charges[category] = sum(amount_charges)
            except TypeError:
                pass
        return purchase_charges
