"""
The file contains the parsing functions for JetBlue airline emails.
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
import logging
import requests
from bs4 import BeautifulSoup

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.jetblue import JETBLUE_RULE, \
    JETBLUE_RULE_ORDER, SEGMENT_IDENTIFY_RULES, FORWARD_TRAVERSAL_LIMIT, DATE_FORMAT, \
    FLIGHT_CHARGES_LINK_REGEX, RECORD_CONFIRMATIONS_REGEX, CHARGE_PATTERN, REWARD_PROGRAM
from lola.itinerary_parsing.utils import get_matched_text

logger = logging.getLogger('lola.airline')


@travel_filter(source='JetBlue', sender='omair.shamshir@arbisoft.com',
               subject=['Itinerary for your upcoming trip', 'Your itinerary for your upcoming trip'])
@travel_filter(source='JetBlue', sender='jetblueairways@email.jetblue.com',
               subject=['Itinerary for your upcoming trip', 'Your itinerary for your upcoming trip'])
@travel_filter(source='JetBlue', sender='reservations@jetblue.com',
               subject=['Itinerary for your upcoming trip', 'Your itinerary for your upcoming trip'])
@travel_filter(source='JetBlue', sender='rachel.neasham@gmail.com', subject=['Check in for your flight to'])
# pylint: disable=no-self-use
class JetBlueFilter(AirlineFilter):
    """
    Base class for JetBlue airline emails parsing. Function are overriding in this class
    """

    def __init__(self):
        super(JetBlueFilter, self).__init__()
        self.airline_rules = JETBLUE_RULE
        self.airline_rules_order = JETBLUE_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently 'JetBlue' is returning
        """
        return 'JetBlue'

    def find_flight_charges_detail_text(self, soup):
        """
        Parse flight charges from external link
        """
        # most jetblue emails contain a link to a
        # "detailed receipt" which we can recursively parse.
        # they don't always work, though.
        # e.g. https://www.virtuallythere.com/new/eticketPrint.html?pnr=EK5MELFJGU4D&pcc=A0A0&aaaCity=OFC&language=0&name=ENGLISH&host=B6&ETNBR1=2792123976134&ETSTC1=null&ETDTE1=09JAN&ETISS1=OFC5G8I&ETTOT=1&SCOS=false&action=printEticket
        flight_charges_link = soup.find('a', {'href': FLIGHT_CHARGES_LINK_REGEX})
        if flight_charges_link:
            try:
                response = requests.get(flight_charges_link.get('href'))
                return response.text
            except Exception as error:
                logger.error('Exception while parsing flight charges detail in jetblue airline %s',
                                 error.message)

    @overrides(AirlineFilter)
    def parse_record_locator(self, email, soup):
        """
        Parse confirmation number
        """
        confirmation_match = RECORD_CONFIRMATIONS_REGEX.search(soup.text)
        if confirmation_match:
            return confirmation_match.group(1).strip()
        return super(JetBlueFilter, self).parse_record_locator(email, soup)

    @overrides(AirlineFilter)
    def get_charges_pattern(self):
        return CHARGE_PATTERN

    @overrides(AirlineFilter)
    def find_purchase_charges_breakup(self, email, soup, currency_code):
        text_response = self.find_flight_charges_detail_text(soup)
        if text_response:
            soup = BeautifulSoup(text_response, self.parser_class())
        return super(JetBlueFilter, self).find_purchase_charges_breakup(email, soup, currency_code)

    @overrides(AirlineFilter)
    def enhance_flight_info(self, segments, flight_info, value, soup):
        if flight_info == 'rewards_program':
            res = soup.findAll(text=REWARD_PROGRAM)
            if res:
                segments.update({'rewards_program': get_matched_text(res)})
