"""
The file contains the parsing/overrides functions for Southwest airline emails.

"""
from __future__ import absolute_import
import logging

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.southwest import SOUTHWEST_RULE, SOUTHWEST_RULE_ORDER, \
    DATE_FORMAT, SEGMENT_IDENTIFY_RULES, FORWARD_TRAVERSAL_LIMIT, \
    PASSENGER_IDENTIFY_RULE
from lola.itinerary_parsing.utils import find_target, get_next_sibling_target, \
    get_node_contents

logger = logging.getLogger('lola.airline')


@travel_filter(source='Southwest Airlines', sender='SouthwestAirlines@luv.southwest.com',
               subject=['Flight reservation'])
@travel_filter(source='Southwest Airlines', sender='doug.morgan@gmail.com', subject=['Flight reservation'])
# pylint: disable=no-self-use
# pylint: disable=C0103
class SouthwestFilter(AirlineFilter):
    """
    Base class for Southwest airline emails parsing.
    Override functions for Southwest airline are in this class
    """

    def __init__(self):
        super(SouthwestFilter, self).__init__()
        self.airline_rules = SOUTHWEST_RULE
        self.airline_rules_order = SOUTHWEST_RULE_ORDER
        self.date_format = DATE_FORMAT
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    @overrides(AirlineFilter)
    def filter_source(self):
        """
        Return the source name of airline and currently 'Southwest Airlines' is returning
        """
        return 'Southwest Airlines'

    @overrides(AirlineFilter)
    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html.parser'
        """
        return "html.parser"

    @overrides(AirlineFilter)
    def enhance_flight_info(self, segments, flight_info, value, soup):
        if flight_info != 'loyalty':
            return

        flight_data = self.get_extra_flight_info(soup)
        flight_dict = {}
        for item in flight_data:
            air_confirmation = item.get('Air Confirmation')
            rapid_rewards = item.get('Rapid Rewards #')

            if flight_dict.get('transaction_id') and air_confirmation:
                flight_dict.get('transaction_id').append(air_confirmation)
            elif air_confirmation:
                flight_dict.update({'transaction_id': [air_confirmation]})
            else:
                air_confirmation = self.parse_record_locator(None, soup)
                if air_confirmation:
                    flight_dict.update({'transaction_id': [air_confirmation]})

            if flight_dict.get('loyalty') and rapid_rewards:
                flight_dict.get('loyalty').append(rapid_rewards)
            elif rapid_rewards:
                flight_dict.update({'loyalty': [rapid_rewards]})

        segments.update(flight_dict)

    def get_extra_flight_info(self, soup):
        """
        Getting transaction id on the behalf Passenger segment identify pattern
        """
        flight_info = []
        # Find passenger heading table and then get its nodes contents in a list
        passenger_target = find_target(soup, PASSENGER_IDENTIFY_RULE)
        passenger_table_heading = get_node_contents(passenger_target)
        while passenger_target:
            passenger_target = get_next_sibling_target(passenger_target, False)
            # Parse passenger names till we get flight segment table heading text
            if 'date' in passenger_target.text.lower():
                break
                # get passenger contents in a list
            passenger_contents = get_node_contents(passenger_target)
            flight_info.append(dict(zip(passenger_table_heading, passenger_contents)))

        return flight_info
