"""
The file contains the parsing/overrides functions for Virgin America airline emails.
"""
from __future__ import absolute_import
import logging
from itertools import chain

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers import travel_filter
from lola.itinerary_parsing.parsers.email_parsers.airlines import AirlineFilter
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines.virgin import VIRGIN_AMERICA_RULE, \
    VIRGIN_AMERICA_RULE_ORDER, DATE_FORMAT, SEGMENT_IDENTIFY_RULES, FORWARD_TRAVERSAL_LIMIT

logger = logging.getLogger('lola.airline')


@travel_filter(source='Virgin America', sender='virginamerica@elevate.virginamerica.com',
               subject=['Virgin America Reservation'])
@travel_filter(source='Virgin America', sender='leandra@lolatravel.com', subject=['Virgin America Reservation'])
@travel_filter(source='Virgin America', sender='rg.vanessa@gmail.com', subject=['Virgin America Reservation'])
# pylint: disable=no-self-use
class VirginAmericaFilter(AirlineFilter):
    """
     Base class for Virgin America airline emails parsing.
    Override functions for Virgin America airline are in this class
    """

    def __init__(self):
        super(VirginAmericaFilter, self).__init__()
        self.airline_rules = VIRGIN_AMERICA_RULE
        self.airline_rules_order = VIRGIN_AMERICA_RULE_ORDER
        self.segment_identify_rule = SEGMENT_IDENTIFY_RULES
        self.date_format = DATE_FORMAT
        self.forward_traversal_limit = FORWARD_TRAVERSAL_LIMIT

    @overrides(AirlineFilter)
    def filter_source(self):
        """
         Return the source name of airline and currently 'Virgin America Airlines' is returning
        """
        return 'Virgin America'

    @overrides(AirlineFilter)
    def parser_class(self):
        """
        Return class of the parser, currently this returns 'html.parser'
        """
        return 'html.parser'

    @overrides(AirlineFilter)
    def sum_purchase_amounts(self, charges_keys_to_values_dict):
        # There are multiple values in base category. Get minimum value that represent the actual base value
        try:
            base_values = charges_keys_to_values_dict.get('base').values()
            base_value = min(list(chain.from_iterable(base_values)))
            charges_keys_to_values_dict['base'] = {'base': [base_value]}
        except (TypeError, AttributeError):
            pass

        return super(VirginAmericaFilter, self).sum_purchase_amounts(charges_keys_to_values_dict)
