"""
This file contains the base class for hotel
filters.
"""

from __future__ import absolute_import

import logging
import re

from bs4 import BeautifulSoup, Tag

from lola.itinerary_parsing.utils import overrides
from lola.itinerary_parsing.parsers.email_parsers.rules.hotels import CHARGE_PATTERN, PATTERN_PARTICIPANTS, PATTERN_ADULT, \
    PATTERN_CHILD, PATTERN_AMENITIES, CATEGORY_BED, \
    PATTERN_BED1, PATTERN_BED2, PATTERN_BED3, PATTERN_RATING, \
    PATTERN_ROOMS, PATTERN_URL, HOTEL_PATTERNS, HOTEL_ADDRESS_PATTERN, PATTERNS_GUESTS, \
    PATTERNS_ADULTS, PATTERNS_CHILDREN, PATTERNS_NAME, HOTEL_NAME_PATTERNS, \
    ARRIVAL_PATTERNS, DEPARTURE_PATTERNS, PATTERNS_ARRIVAL_DATE, \
    PATTERNS_DEPARTURE_DATE, PATTERNS_ROOMS, PATTERNS_REFUNDABLE, \
    PATTERNS_NONSMOKING, TOKENS_ADDRESS, PATTERN_ADULT2, \
    PATTERN_CHILD2, TOKEN_NONREFUNDABLE, DEFAULT_ARRIVAL_TIME, \
    DEFAULT_DEPARTURE_TIME, LABEL_AMENITIES, PATTERN_DATE, PATTERN_TIME_IN_DATE, ILLEGAL_STR_AMENITIES, PATTERN_PARTICIPANTS2, \
    DISTANCE, ILLEGAL_HOTEL_TITLE_TOKEN
from lola.itinerary_parsing.parsers.email_parsers import TravelFilter
from lola.itinerary_parsing.utils import parse_email_information, convert_datetime_to_timestamp, \
    convert_to_str, remove_empty_childs, extract_info_from_address, \
    remove_element_from_soup, get_node_position_under_parent

__all__ = ['wyndham', 'starwood', 'hyatt', 'hilton', 'kayak', 'ihg', 'sheraton', 'worldwide',
           'choicehotels', 'bestwestern', 'hoteltonight']

logger = logging.getLogger('lola.hotel')

# pylint: disable=no-self-use
class HotelFilter(TravelFilter):
    """
    This is the Base class for hotel emails parser.
    All hotel parsers should be inherited from
    this.
    """

    def __init__(self):
        super(HotelFilter, self).__init__()
        self.multiline = False
        self.is_forward = True

    @overrides(TravelFilter)
    def filter_source(self):
        return 'Generic Hotel'

    @overrides(TravelFilter)
    def parse_message(self, email):
        """
        Extract information out of the email
        """
        text = email.html
        if not text or text == '':
            text = email.body

        soup = BeautifulSoup(text, "html5lib")

        properties = {}

        self.find_keywords_under_categories(soup, self.hotel_patterns(), properties)

        properties['user_id'] = email.sender

        properties['raw_email'] = parse_email_information(email)

        properties['participants'] = self.parse_participants(email, soup)

        num_adult, num_child = self.parse_guests(email, soup, properties)
        properties['num_adults'] = num_adult
        properties['num_children'] = num_child

        # We haven't found any sample email that contains time when the
        # booking has actually been confirmed
        properties['transaction_time'] = None

        properties['provider_name'] = self.extract_hotel_name(email, soup)
        logger.info("Parsing email for provider: %s", properties['provider_name'])
        hotel_information = {}
        self.extract_all_hotel_information(soup, email, hotel_information, properties)

        properties['hotel'] = hotel_information
        properties['flight'] = None
        properties['rewards_program'] = self.extract_reward_program(email, soup)
        properties['rewards_id'] = self.extract_reward_id(soup)
        # We haven't found provider code so far, therefore it is none for now
        properties['provider_code'] = None

        properties.pop('_guests', None)
        properties.pop('_room_type', None)
        return properties

    def extract_all_hotel_information(self, soup, email, hotel_information, properties):
        """
        Extract all information from email that lies at
        hotel level in json schema
        """
        hotel_name = properties.get('provider_name', None)
        hotel_information['name'] = hotel_name

        hotel_address = self.extract_hotel_address(soup, properties, hotel_name)
        hotel_information['address1'] = hotel_address
        address_info = None
        if hotel_address:
            address_info = extract_info_from_address(hotel_address)

        hotel_information['city'] = None
        hotel_information['province'] = address_info.state if address_info else None
        hotel_information['postal_code'] = address_info.zip if address_info else None

        location_components = self.hotel_service.\
            parse_address(hotel_information.get('address1', None))

        hotel_information['country_code'] = location_components.get('address')\
            .get('country_code')
        coordinates = location_components.get('coordinates')
        hotel_information['longitude'] = coordinates.get('lng')
        hotel_information['latitude'] = coordinates.get('lat')

        currency_code = self.extract_currency_code(soup)
        hotel_information['currency_code'] = currency_code
        hotel_information['non_refundable'] = self.is_non_refundable(soup)
        hotel_information['website_url'] = self.extract_hotel_url(soup, properties)
        hotel_information['thumbnail_url'] = self.extract_hotel_image_url(soup, properties)
        hotel_information['num_rooms'] = self.extract_number_of_booked_rooms(email, soup)
        hotel_information['description'] = self.extract_hotel_description(soup, hotel_name)

        adults = properties.get('num_adults', 0)
        children = properties.get('num_children', 0)

        num_guest = adults if adults else 0
        num_guest += children if children else 0

        try:
            if num_guest > 0:
                hotel_information['num_guests'] = num_guest
            elif properties.get('_guests'):
                hotel_information['num_guests'] = int(properties.get('_guests'))
        except ValueError:
            hotel_information['num_guests'] = None
        # todo: Right now we are getting single transaction id but we need to get multiple transaction id
        transaction_id = self.parse_record_locator(email, soup)
        properties['transaction_id'] = [transaction_id] if transaction_id else []

        arrival, departure = self.parse_arrival_and_departure(email, soup, self.multiline)

        hotel_information['amenities'] = self.parse_amenities(soup, properties)

        try:
            hotel_information['checkin_date'] = convert_datetime_to_timestamp(str(arrival)) if arrival else None
            hotel_information['checkout_date'] = convert_datetime_to_timestamp(str(departure)) if departure else None
        except ValueError, error:
            logger.exception('Error while converting date to timestamp: %s', error.message)

        if not properties:
            return []

        self.parse_smoking(email, hotel_information)
        self.parse_bedding(email, properties, hotel_information, soup)
        hotel_information['star_rating'] = self.parse_rating(email)

        cost_dollar, orignal_currency = self.find_purchase_charges_breakup(email, soup, currency_code)

        properties['cost_dollars'] = cost_dollar
        hotel_information['base_rate'] = orignal_currency['base']
        hotel_information['tax_rate'] = orignal_currency['tax']

        hotel_information['high_rate'] = None
        hotel_information['low_rate'] = None
        # We have parsed direct email so far, therefore following fields are None for now
        hotel_information['supplier_type'] = None
        hotel_information['trip_advisor_count'] = None
        hotel_information['trip_advisor_rating'] = None
        hotel_information['trip_advisor_url'] = None

        # We haven't got any email so far which holds information for on site purchase
        hotel_information['onsite_purchases'] = {'cost': None, 'item': None}


    def find_keywords_under_categories(self, soup, patterns, properties):
        """
        Function tries to find specific keywords under
        categories e.g _guests, _room_type
        """
        for key, triggers in patterns.iteritems():
            for trigger in triggers:
                if properties.get(key):
                    break
                if trigger == 'Guests':
                    pass
                chunk = self.parse_specific_tokens(soup, trigger, self.multiline)
                if chunk:
                    properties[key] = chunk.strip()
                    break

    def is_non_refundable(self, soup):
        """
        Functions decide whether hotels follows
        non-refundable policy or not
        """
        for pattern in PATTERNS_REFUNDABLE:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=True, include_anchors=False)
            if chunk:
                for key in TOKEN_NONREFUNDABLE.keys():
                    if key.lower() in chunk.lower():
                        return TOKEN_NONREFUNDABLE[key]
        return False

    def extract_hotel_url(self, soup, properties):
        """
        Extract url to hotel
        """
        match = soup.find(text=PATTERN_URL)

        def traverse_parent(node):
            """
            Moves one level up in the hierarchy
            until we reached to element which is
            an achor tag
            """
            if node.name == 'a':
                return str(node.attrs['href'].strip())
            if node.parent:
                return traverse_parent(node.parent)

        if match:
            return traverse_parent(match)
        # If we still not able to extract hotel url try getting it from anchor tag which contains text
        # equal to hotel name
        if properties.get('provider_name'):
            all_anchor = soup.findAll('a', text=re.compile(properties.get('provider_name')))
            if all_anchor:
                for anchor in all_anchor:
                    text = anchor.getText(separator=u' ')
                    if text == properties.get('provider_name'):
                        return str(anchor.attrs['href'].strip())

        # Try hard to find hotel url from anchor tag that contains img tag as a hotel logo into it
        all_anchor = soup.findAll('a')
        for anchor in all_anchor:
            if bool(anchor.find('img')) and len(anchor.contents) == 1:
                return str(anchor.attrs['href'].strip())
        return None

    def extract_hotel_image_url(self, soup, properties):
        """
        This functions tries to find closest image to hotel name
        and we are assuming closest image will be a image of hotel.
        """
        hotel_name = properties.get('provider_name')
        if hotel_name:
            to_be_removed = soup.findAll('img', {'width': 1, 'height': 1})
            remove_element_from_soup(to_be_removed)

            matched_result = soup.find(text=re.compile(r'^\s*%s\s*$' % hotel_name))
            if not matched_result:
                hotel_name_parts = hotel_name.split()
                results = None
                for item in hotel_name_parts:
                    results = soup.find(text=re.compile(r'^\s*%s\s*$' % item))
                    if results:
                        matched_result = results
                        break
            try:
                while matched_result:
                    if matched_result.next_sibling or matched_result.previous_sibling:
                        parent = matched_result.parent
                        img = parent.find('img')

                        if img and 'src' in img.attrs:
                            src_val = img['src'].lower()
                            if not src_val.endswith('jpg') or \
                                    bool(set(['shade']) & set(re.split(r'/|_', src_val))):
                                img.extract()
                                continue
                            return str(img['src'])
                    matched_result = matched_result.parent
            except AttributeError, error:
                logger.exception('Error while extracting hotel image url for %s: %s', hotel_name, error.message)
        return None

    def extract_number_of_booked_rooms(self, email, soup):
        """
        Parse number of rooms booked
        Function is expected to return int type value
        """
        for pattern in PATTERNS_ROOMS:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=False)
            if chunk:
                try:
                    return int(chunk)
                except ValueError:
                    pass

        text = email.body or email.html
        match = PATTERN_ROOMS.findall(text)
        if match:
            try:
                return int(match[0].strip())
            except ValueError, error:
                logger.exception('Error while parsing number of rooms booked: %s', error.message)
                return None
        return None

    def parse_rating(self, email):
        """
        Extract hotel rating if given in email
        """
        text = email.body or email.html
        match = PATTERN_RATING.search(text)
        if match:
            return float(match.group(1).strip())
        return None

    def parse_specific_tokens(self, soup, pattern, multiline=False):
        """
        This method looks for content in email against specific pattern under
        categories
        """
        return self.extract_contents_with_label(soup, pattern, multiline=multiline)

    def parse_arrival_and_departure(self, email, soup, multiline=False):
        """
        Extract arrival and departure date
        """
        arrival = None
        departure = None

        def process_arrival_departure_text(arrival, departure, matched_patterns):
            """
            Function makes an analysis of the text for Arrival/Departure Date and make sure
            if time is combine with the date so that we can avoid looking for time individually
            """
            if PATTERN_TIME_IN_DATE.search(arrival) and PATTERN_TIME_IN_DATE.search(departure):
                arrival = extract_date(arrival)
                departure = extract_date(departure)
            else:
                # We have certain example in which time and date (Arrival/Departure) are given separately.
                # There might be a case when we could get same value for date and time since we have
                # some common label patterns for searching content for arrival and departure. Need
                # to avoid mentioned scenario
                arrival_time, departure_time = self.parse_arrival_departure_time\
                    (soup, matched_patterns, multiline=True)
                if arrival_time:
                    arrival = extract_date('%s %s' % (arrival, arrival_time))
                else:
                    arrival = extract_date('%s %s' % (arrival, DEFAULT_ARRIVAL_TIME))
                if departure_time:
                    departure = extract_date('%s %s' % (departure, departure_time))
                else:
                    departure = extract_date('%s %s' % (departure, DEFAULT_DEPARTURE_TIME))
            return arrival, departure

        def extract_date(text):
            """
            Extract date looking content out of the
            text
            """
            from lola.itinerary_parsing.utils import find_date_in_string
            return find_date_in_string(text, default=email.sent_at)

        matched_patterns = []
        for pattern in PATTERNS_ARRIVAL_DATE:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=multiline)
            if chunk:
                arrival = chunk
                matched_patterns.append(pattern)
                break

        if arrival:
            arrival_date_parts = arrival.split()
            result = PATTERN_DATE.findall(arrival)

            if len(arrival_date_parts) != len(set(arrival_date_parts)) \
                    or (result and len(result[0]) > 1):

                if u'\u2013' in arrival or '-' in arrival:
                    arrival, departure = re.split(u'\u2013|-', arrival)
                    if arrival and departure:
                        return process_arrival_departure_text(arrival, departure, matched_patterns)

        for pattern in PATTERNS_DEPARTURE_DATE:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=multiline)
            if chunk:
                departure = chunk
                matched_patterns.append(pattern)
                break

        if arrival and departure:
            # Before getting into basic search, look for specific token in arrival and departure date
            arrival_date_parts = re.split(r'After|Before', arrival, flags=re.IGNORECASE)
            departure_date_parts = re.split(r'After|Before', departure, flags=re.IGNORECASE)

            if len(arrival_date_parts) > 1 and len(departure_date_parts) > 1:
                arrival = extract_date('%s %s' % (arrival_date_parts[0], arrival_date_parts[1]))
                departure = extract_date('%s %s' % (departure_date_parts[0], departure_date_parts[1]))
                return arrival, departure
            else:
                return process_arrival_departure_text(arrival, departure, matched_patterns)
        return arrival, departure

    def parse_arrival_departure_time(self, soup, matched_pattern, multiline=False):
        """
        There are cases when time for departure and arrival
        is given separately from date therefore adding separate
        function to deal with given scenario.
        :param matched_pattern: Since there are some common patterns to find content for
         Arrival/Departure Date and Arrival/Departure Time therefore matched pattern contains pattern
         for which we have found Arrival/Departure Date for this particular email. This parameter help us to avoid
         looking again for the content against any label for which we already have content.
        """
        arrival_time = None
        departure_time = None

        if not set(matched_pattern) & set(ARRIVAL_PATTERNS):
            for pattern in ARRIVAL_PATTERNS:
                chunk = self.extract_contents_with_label(soup, pattern, multiline=multiline)
                if chunk:
                    arrival_time = chunk
                    break
        else:
            arrival_time = None

        if not set(matched_pattern) & set(DEPARTURE_PATTERNS):
            for pattern in DEPARTURE_PATTERNS:
                chunk = self.extract_contents_with_label(soup, pattern, multiline=multiline)
                if chunk:
                    departure_time = chunk
                    break
        else:
            departure_time = None
        return arrival_time, departure_time

    def parse_bedding(self, email, properties, hotel_information, soup):
        """
        Extract bedding information
        """
        if 'bed_type' not in properties:
            # if we didn't figure out accommodations, try to
            # infer it from room type or preferences
            room_type = properties.get('_room_type', "").lower()
            preferences = properties.get('_preferences', "").lower()

            all_text = room_type + preferences

            # strip out "smoking"

            while "smoking" in all_text:
                pos = all_text.find("smoking")
                all_text = all_text[0:pos] + all_text[pos + 7:]

            bed_number = None
            bed_type = None

            match = PATTERN_BED1.search(all_text)
            if match:
                bed_type = match.group(2)
                bed_number = match.group(1) or '1'
            else:
                match = PATTERN_BED3.search(soup.getText(separator=u' '))
                if match:
                    bed_type = match.group(2)
                    bed_number = match.group(1) or '1'
                else:
                    match = PATTERN_BED2.search(all_text)
                    if match:
                        bed_type = match.group(1)
                        bed_number = 1
                    else:
                        match = PATTERN_BED2.search(soup.getText(separator=u' '))
                        if match:
                            bed_type = match.group(1)
                            bed_number = 1

            if not bed_type:
                # ok, try harder to find the important phrases in the body
                text = email.html or email.body
                if text:
                    text = text.lower()
                    for key, fragments in CATEGORY_BED.iteritems():
                        if bed_type:
                            break
                        for fragment in fragments:
                            if fragment in text:
                                bed_type = key
                                break
            room_information = {}
            # Room number is none because we haven't found it in email so far.
            room_information = {'number': None}
            # samples:
            # " 2 Grand Double Beds"
            # "1 king bed:Corner Room"
            # "1 KING 1 BEDRM STE NONSMOK"
            # "2 QUEEN BEDS NONSMOKING"
            # "2 DOUBLE BEDS NONSMOKING"
            # "STANDARD RM QUEEN BED"
            # "1 King, Sofa Bed, Mini-Fridge, 300Sqft/27Sqm, Living/Sitting Area,
            #  Wireless Internet, Complimentary, Coffee/Tea Maker"
            # "Non Smoking, King Bed"

            try:
                room_information['bed_number'] = int(bed_number) if bed_number else None
            except ValueError, error:
                logger.exception('Error while parsing bed number: %s', error.message)
                room_information['bed_number'] = None

            room_information['bed_type'] = bed_type.lower().strip() if bed_type else None
            hotel_information['rooms'] = [room_information]

    def parse_amenities(self, soup, properties):
        """
        Extract amenities from email if given
        """
        def is_illegal_token(str):
            if set(ILLEGAL_STR_AMENITIES) & set(str.lower().split()):
                return True
            return False

        room_type = None
        if '_room_type' in properties:
            room_type = properties.get('_room_type')
            match = PATTERN_AMENITIES.search(room_type)

            if match:
                amenities = match.group(1).strip()
                if is_illegal_token(amenities):
                    return None
                amenities = re.split(r'(?:\,|\band\b|\s{2,}|:)', amenities)
                if len(amenities) > 1:
                    return amenities
            LABEL_AMENITIES.append(room_type)

        # Try to search amenities by label
        for pattern in LABEL_AMENITIES:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=self.multiline)
            if chunk:
                amenities = chunk.strip()
                if is_illegal_token(amenities):
                    return None
                return re.split(r'(?:\,|\band\b|\s{2,}|:)', amenities)
        return None

    # pylint: disable=unused-argument
    def extract_reward_program(self, email, soup):
        """
        This function used to extract reward program.
        Since it is empty for base class because till now
        we have got reward program for hilton hotel only
        """
        return None

    def extract_reward_id(self, soup):
        """
        Extract reward id against any reward program
        """
        patterns = ['Hilton HHonors', r'Hilton HHonors\s?(Account)?']
        for pattern in patterns:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=True, expected_type=int)
            if chunk:
                return chunk
        return None

    def parse_smoking(self, email, properties):
        """
        Check whether smoking is allowed or not in the hotel room
        """

        properties['smoking'] = True
        # if we didn't figure out smoking vs. non-smoking, try to infer it from room type or preferences
        room_type = properties.get('_room_type', "").lower()
        preferences = properties.get('_preferences', "").lower()

        for pattern in PATTERNS_NONSMOKING:
            if pattern in room_type or pattern in preferences:
                properties['smoking'] = False
                break

        text = email.html or email.body
        text = text.lower()
        for pattern in PATTERNS_NONSMOKING:
            if not properties['smoking']:
                break
            if pattern in text:
                properties['smoking'] = False
                break

    def extract_hotel_name(self, email, soup):
        """
        Extract hotel name
        """
        hotel_name = None
        for trigger in HOTEL_NAME_PATTERNS:
            chunk = self.extract_contents_with_label(soup, trigger, multiline=False)
            if chunk:
                hotel_name = chunk
                break

        for subject_pattern in PATTERNS_NAME:
            pattern = re.compile(subject_pattern)
            match = pattern.search(email.subject)
            if match:
                hotel_name = match.group(1).strip()
                anchors = soup.findAll('a', text=re.compile(hotel_name))
                if anchors:
                    for anchor in anchors:
                        if 'style' in anchor.attrs and 'bold'in anchor.attrs['style']:
                            hotel_name = anchor.text.strip()
                            break

        def collect_sib_text(node):
            """
            This function start looking for hotel name element
            starting from element we got from matched pattern.
            Keep on traversing parent of matched element until
            we found previous sibling for it.
            Than finds and anchor element or strong element because
            mostly name for hotels are under anchor or strong elements.
            """
            hotel_title = None
            remove_empty_childs(node.parent)

            if not node.previous_sibling:
                return collect_sib_text(node.parent)

            try:
                # Expected element which can hold value for hotel name
                elements = ['a', 'strong', 'font', 'h4', 'h3', 'h2', 'b']
                for element in elements:
                    if isinstance(element, dict):
                        results = node.parent.findAll(attrs = {element.keys()[0]: re.compile(element[element.keys()[0]])})
                    else:
                        results = node.parent.findAll(element)

                    for res in results:
                        searched_node_position = node.parent.index(node)
                        hotel_element_position, hotel_element = get_node_position_under_parent(node.parent, res)

                        if hotel_element_position >= searched_node_position:
                            break

                        # Excluding br elements to counts towards actual no of children for a
                        # parent
                        br_counts = None
                        if node.find_previous_sibling('br'):
                            br_counts = len(node.find_previous_siblings('br'))
                            if searched_node_position - br_counts > 0:
                                searched_node_position -= br_counts

                        if hotel_element.find_previous_sibling('br'):
                            br_counts = len(hotel_element.find_previous_siblings('br'))
                            if hotel_element_position - br_counts > 0:
                                hotel_element_position -= br_counts

                        # Added validation check for hotel name
                        # Value 4 is added to determine distance between searched node and node that
                        # contains name if it is greater than 4 than it might be a wrong value for
                        # hotel name. Value for hotel name is expected to lies at distance not
                        # more than 4 from node that holds value for contact number (phone #)
                        if res.text and not res.text.isspace() \
                                and (searched_node_position - hotel_element_position) <= DISTANCE:
                            hotel_title = res.text.strip()
                            if hotel_title.endswith(':') or \
                                    ILLEGAL_HOTEL_TITLE_TOKEN.search(hotel_title):
                                continue
                            return hotel_title
                        continue

                if not hotel_title:
                    return collect_sib_text(node.parent)

                return None

            except (AttributeError, IndexError):
                return None

        style_tag = soup.find('style');
        if style_tag:
            style_tag.extract()

        if not hotel_name:
            nodes = soup.findAll(text=re.compile(r'(%s)' % ('|'.join(TOKENS_ADDRESS))))
            if not nodes:
                nodes = soup.findAll('a', {'href': re.compile(r'(%s)' % ('|'.join(TOKENS_ADDRESS)), re.IGNORECASE)})
            if nodes:
                for node in nodes:
                    hotel_name = collect_sib_text(node)
                    if hotel_name and not hotel_name.isspace():
                        break

        if hotel_name:
            hotel_name = re.sub(r'[\n\r\t]', '', hotel_name)
        return hotel_name

    def parse_guests(self, email, soup, properties):
        """
        Extract info about guest
        """
        adult = None
        child = None

        if properties.get('_guests'):
            match = PATTERN_ADULT2.match(properties.get('_guests'))
            if match:
                adult = match.group(1).strip()
            match = PATTERN_CHILD2.match(properties.get('_guests'))
            if match:
                child = match.group(1).strip()

        # Try once more if we have following patterns for label are available

        if not adult:
            for pattern in PATTERNS_ADULTS:
                chunk = self.extract_contents_with_label(soup, pattern, multiline=False)
                if chunk:
                    adult = chunk

        if not child:
            for pattern in PATTERNS_CHILDREN:
                chunk = self.extract_contents_with_label(soup, pattern, multiline=False)
                if chunk:
                    child = chunk

        def find_pattern(text, pattern):
            """
            Finds specific pattern out of the email
            """
            match = pattern.findall(text)
            if match:
                return match[0]
            return None

        if not adult:
            text = email.body or email.html
            adult = find_pattern(text, PATTERN_ADULT)

        if not child:
            text = email.body or email.html
            child = find_pattern(text, PATTERN_CHILD)
        try:
            adult_val = None
            child_val = None
            if adult:
                adult_val = int(adult)
                adult = adult_val
            if child:
                child_val = int(child)
                child = child_val
        except (ValueError, TypeError) as error:
            logger.exception('Error while parsing guests %s', error.message)
            return adult_val, child_val

        return adult, child

    def parse_participants(self, email, soup):
        """
        Extract info about the participants this email
        is booking something for
        """
        participant_name = None
        for pattern in PATTERNS_GUESTS:
            chunk = self.extract_contents_with_label(soup, pattern, multiline=False)
            if chunk:
                participant_name = convert_to_str(chunk)
                break

        if not participant_name:
            text = soup.getText(separator='\n')
            match = PATTERN_PARTICIPANTS.search(text)

            if not match:
                match = PATTERN_PARTICIPANTS2.search(text)
                if match:
                    participant_name = match.group(1).strip()
            else:
                participant_name = match.group(1).strip()

        if participant_name:
            return [' '.join(participant_name.split())]
        return None

    def remove_noise_token_from_address(self, address):
        """
        Removes if there exist any noise tokens in
        hotel address
        """
        address = address.replace('|', ',')
        for pattern in TOKENS_ADDRESS:
            if pattern in address:
                address = address[:address.index(pattern)]
        patterns = re.compile(r'(.*)(?:\s+\[+)', re.UNICODE)
        match = patterns.search(address)
        if match:
            address = match.group(1).strip()
        return ' '.join(address.split())

    # pylint: disable=unused-argument
    def parse_rewards_account(self, email, soup, properties):
        """
        Parse rewards account, since this function is
        empty for base class because till now we have only
        found rewards account for hilton hotel
        """
        return {}

    # pylint: disable=unused-argument
    def extract_hotel_address(self, soup, properties, hotel_name):
        """
        Extract hotel address
        """
        for trigger in HOTEL_ADDRESS_PATTERN:
            chunk = self.extract_contents_with_label(soup, trigger, multiline=True)
            if chunk:
                return chunk
        name = properties.get('provider_name')
        if name:
            # now use the base logic to extract the blob data for the hotel name
            address = self.extract_contents_with_label(soup, name, multiline=True, include_anchors=True)
            if address:
                return self.remove_noise_token_from_address(address)

        # Try to find hotel address before the node that contains token like Phone: Ph: Fax: etc
        def collect_sib_text(node):
            remove_empty_childs(node.parent)
            if node.previous_sibling:
                collected_text = ''
                for sib in node.previous_siblings:
                    if isinstance(sib, Tag):
                        if collected_text and (sib.name in ['img', 'h1', 'a', 'strong'])\
                                or sib.find('img'):
                            return collected_text
                        sib = sib.text
                    collected_text = '%s %s' % (sib.strip(), collected_text)
                return collected_text
            else:
                return collect_sib_text(node.parent)

        node = soup.find(text=re.compile(r'(%s)+?' % ('|'.join(TOKENS_ADDRESS))))
        if node:
            hotel_address = collect_sib_text(node)
            if hotel_address:
                return self.remove_noise_token_from_address(hotel_address)
        return None

    # pylint: disable=unused-argument
    def extract_hotel_description(self, soup, hotel_name):
        """
        Extract hotel description if given
        """
        return None

    def hotel_patterns(self):
        """
        Patterns which are used to find particular
        label under differnt category
        """
        return HOTEL_PATTERNS

    @overrides(TravelFilter)
    def sum_purchase_amounts(self, charges_keys_to_values_dict):
        charges = {}
        for category, charge in charges_keys_to_values_dict.iteritems():
            purchase_values = []
            for key, val in charge.iteritems():
                purchase_values.append(max(list(set(val))))
            if category == 'total' or category == 'base' and len(purchase_values) > 1:
                charges[category] = max(purchase_values)
            else:
                charges[category] = sum(purchase_values)
        return charges

    @overrides(TravelFilter)
    def get_charges_pattern(self):
        return CHARGE_PATTERN
