"""
This file contains regular expression  that are using in Air Canada airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment

# To check if there is some flight operator name in row
# eg:Flight AC5585 is operated by United Airlines
FLIGHT_OPERATOR_REGEX = re.compile(r"""
    (?xu)                   # verbose regex, unicode character
    (?:                     # Non-capturing
    is\s+operated\s+by\s*)   # match string "is operated by "
    ([A-z\s]+)\.          # match all character in range A to Z and a to z or match any white space character
    """)

# To check flight departure and arrival city pattern
# eg: San Francisco, San Francisco Int'l (SFO)
AIRPORT_REGEX = re.compile(r"""
                (?x)
                ([A-z\-\'\.\s]+\s*\([A-Z]{3}\))
    """)

AIRPORT_CODE = re.compile(r"""
                (?x) # Example: (SFO)
                \(([A-Z]{3})\)
    """)

# To parse passenger name with regex
# eg: 1: Ms Leanne Blade :
PASSENGER_REGEX = re.compile(r"""
    (?x)           # verbose regex
    (?:\d+:\s*)    # Non-capturing group (match a digit follow by : and zero or more white space)
    ([A-z|\s]*)    # match all character in range A to Z and a to z or match any white space character
    \s+:           # match one or more white space follow by :
    """)

# To get rewards id (frequent flyer number)
REWARD_ID_REGEX = re.compile(r"""
    (?xiu)           # verbose regex
    (?:frequent\s*flyer\s*prog\s*:)    # Non-capturing group (match a word 'Frequent Flyer Prog' follow by : and zero or more white space)
    \s*              # zero or more whitespace character
    (.*)             # matches any character (except newline)
    """)

FLIGHT_NUMBER_REGEX = re.compile(r"""
                                    (?x) # Example: AC567
                                    (
                                        AC\d+\b
                                    )
                                    """)
DATE_REGEX = re.compile(r"""(?x) # Example: Mon 22-Feb 2016
                            (
                                [A-z]{3}\s*
                                \d{2}
                                \s*\-\s*
                                [A-z]{3}\s*\d{4}
                            )
                        """)

PLANE_TYPE = re.compile(r"""
                            (?x) # Example: 320
                            (\d{3,})
                            \-?
                            (\d{3,})?
                            """)
LOYALTY = re.compile(r"""(?xi)frequent\ flyer\ prog\s*:\s*(\d+)""")

DATE_FORMAT = ['%a %d-%b %Y']

AIRCANADA_RULES = {
    'airport': {'rule': AIRPORT_REGEX, 'is_repeated': True, 'is_replace': True},
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': True, 'is_replace': False},
    'flight_number': {'rule': FLIGHT_NUMBER_REGEX, 'is_repeated': False, 'is_replace': True},
    'plane_type': {'rule': PLANE_TYPE, 'is_repeated': False, 'is_replace': True},
    'operating_airline': {'rule': FLIGHT_OPERATOR_REGEX, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_REGEX, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGERS_RULE = {
    'name': {'rule': PASSENGER_REGEX, 'is_repeated': True, 'is_replace': True},
    'loyalty': {'rule': LOYALTY, 'is_repeated': False, 'is_replace': True}
}

PASSENGER_SEGMENT = re.compile(r"""(?x)Passenger\ +Information""")

# Purpose of adding list as an item of a list is to make a group, and from a group only one item can be matched
# per email not more than one at the same time.
SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': AIRCANADA_RULES, 'is_validate': True}
                          ], [{'rule': PASSENGER_SEGMENT, 'segment_rule': PASSENGERS_RULE, 'is_validate': False}
                          ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
AIRCANADA_RULES_ORDER = ['name', 'loyalty', 'airport_code', 'airport', 'flight_date', 'time', 'flight_number',
                         'operating_airline', 'plane_type']
