"""
This file contains regular expression  that are using in Air France airlines filter class
"""
import re
# To get flight operator name eg: Operated by: Air France
OPERATOR_REGEX = re.compile(r"""
                            (?x)
                            (?:Operated\ +by\ *:)\s*
                            (.*)   # matches any character (except newline)
    """)

# To get plane type with regex eg: Aircraft : Airbus A319
PLANE_TYPE_REGEX = re.compile(r"""
    (?x)    # verbose regex
    (?:     # Non-capturing
        Aircraft\s*: # match a string 'Aircraft' follow by white space and character :
     )\s*
    (.*)    # matches any character (except newline)
    """)

AIRPORT_CODE = re.compile(r"""(?x)

                          [A-z\s,]+ # match character from one to unlimited times like white space, A to Z and a to z, and ,

                          \s+      # match one or unlimited white space character
                          \(       # match character (
                            ([A-z]+)  # match at least one character(inside bracket) from range A to Z and a to z
                          \)""")

DATE_PATTERN = re.compile(r"""(?x) # Example: Monday, February 1, 2016
                                    (
                                        [A-z]+\s*,\s*[A-z]+\s*\d{1,2}\s*,\s*\d{4}
                                    )
                                """)

DATE_FORMAT = ['%A, %B %d, %Y']


def is_more_expansion_needed_segment(flight_info, is_single):
    airport = flight_info.get('airport_code')
    operating_airline = flight_info.get('operating_airline')
    return not ((airport and len(airport) > 1) and operating_airline)


FLIGHT_NUMBER = re.compile(r"""(?x) # Example: AF1205 - Economy
                            (
                                [A-Z\d]+
                            )
                            \s*\-\s*(?i)
                            (?:economy)
                            """)

SEAT_TYPE = re.compile(r"""(?x) # Example: AF1205 - Economy
                               (?:[A-Z\d]+)
                               \s*\-\s*(?i)
                               (economy)
                           """)

TIME_PATTERN = re.compile(r"""(?xi) # Example: 10:15 AM
                                (?:
                                    [^:\s+]
                                )
                                (
                                    \s+\d{1,2}\s*:\s*\d{1,2}\s*[ap].?[m]
                                )
                                """)

PASSENGER_NAME_REGEX = re.compile(r"""(?x) # Example: BLADE LEANNE
                                            (
                                                [A-Z\\\-]+\s+[A-Z\\\-]+\s*[A-Z\\\-]*
                                            )\s*
                                            (?:Ticket)
                                        """)

AIR_FRANCE_RULES = {
    'airport_code': {'rule': AIRPORT_CODE, 'is_repeated': True, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'operating_airline': {'rule': OPERATOR_REGEX, 'is_repeated': False, 'is_replace': True},
    'plane_type': {'rule': PLANE_TYPE_REGEX, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': False, 'is_replace': True},
    'seat_class': {'rule': SEAT_TYPE, 'is_repeated': False, 'is_replace': False},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGERS_RULE = {
    'name': {'rule': PASSENGER_NAME_REGEX, 'is_repeated': True, 'is_replace': True}
}

PASSENGER_SEGMENT_IDENTIFIER = re.compile(r"""^(?x)Passengers?""")

SEGMENT_IDENTIFY_RULES = [[{'rule': AIRPORT_CODE, 'segment_rule': AIR_FRANCE_RULES, 'is_validate': True}
                          ], [{'rule': PASSENGER_SEGMENT_IDENTIFIER, 'segment_rule': PASSENGERS_RULE,
                               'is_validate': False}, ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
AIR_FRANCE_RULE_ORDER = ['name', 'airport_code', 'flight_date', 'time',
                         'seat_class',
                         'operating_airline', 'flight_number', 'plane_type']
