"""
This file contains regular expression  that are using in delta airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import TIME_PATTERN, is_more_expansion_needed_segment

# To match email subject pattern
TRAVEL_FILTER_REGEX = r'.*\d\d[A-Z][A-Z][A-Z]\d\d$'

# To get flight seat class, flight number and flight name
# eg: DELTA 243 MAIN CABIN (V)
DELTA_REGEX = re.compile(r"""
   (?xu)   # verbose regex,  unicode
     ^     # assert position at start of the string
    ([A-z\s]*) # match zero or more character from range A to Z and a to z or white space
    (\d+)      # match one or more digits
    (\*?)      # match character * optional
    \s*        # match zero or more white space character
    ([A-z\s]*) # match zero or more character from range A to Z and a to z or white space
    (\([A-z]{1,2}\)) # match character(with bracket) from range A to Z and a to z and repeat 1 and 2 times,
    """)


def is_more_expansion_needed(flight_info, is_single):
    return not flight_info


DATE_FORMAT = ['%a, %d%b', '%a %d%b']

# To match passenger names from string like 'ELLEN CHISA SkyMiles #*******811'
PASSENGER_NAME_PATTERN = re.compile(r"""
    (?x)            # verbose regex
    (?:^|NAME)\s*   # Non-capturing group
    (
        [A-Z]+\s    # match more than one character from range [A-Z] follow by white space
        [A-Z ]+     # match more than one character from range [A-Z]
    )
    \s*             # match zero or more white space character
    (?:[A-Z\s]+\s\d{3}|\s+) # Non-capturing group
    """)

# To match airport names from string like 'BOSTON' or 'PARIS-DEGAULLE'
AIRPORT_NAME = re.compile(r"""
    (?x)                # verbose regex
    (
        (?:[A-Z]{3,})\-?,?\s? # match character from [A-Z] and repeat 3 times follow by optional character - or , white space
        (?:[A-Z]{2,})?\s?     # match more than two character from [A-Z] (optional) follow by white space character
        (?:[A-Z]{2,})?\s?     # match more than two character from [A-Z] (optional) follow by white space character
    )
    (?!\([A-Z]{1,2}\)|[a-z]+)   # Negative Lookahead - Assert that it is impossible to match the regex
    """)

# To match flight seat class from string like 'MAIN CABIN (V)'
FLIGHT_CLASS_REGEX = re.compile(r"""
    (?x)                # verbose regex
    ([A-Z ]+)           # match more than one character from range [A-Z] and a space
    (?:\([A-Z]{1,2}\))  # Non-capturing group
    """)

# To match flight number from string like 'DELTA 243'
FLIGHT_NUMBER = re.compile(r"""
    (?x)            # verbose regex
    (
        [A-Z]+      # match more than one character from range [A-Z]
        \ ?         # match a space (optional)
        [A-Z]*\s    # match zero or more character from range [A-Z] follow by white space character
        \d{2,4}     # match digit and repeat 2 to 4 times
    )
    """)

# To match date pattern from string like 'Wed, 24JUN'
DATE_PATTERN = re.compile(r"""
    (?x)            # verbose regex
    (
        [A-Z][a-z]{2}   # match one character from range [A-Z] and two character from range [a-z]
        ,?\s            # match character , (optional) follow by white space character
        \d{2}           # match digit and repeat 2 times
        [A-Z]{3}        # match three character from range [A-Z]
    )
    """)

PASSENGER_IDENTIFY_RULE = re.compile(r"""
    (?x)            # verbose regex
    Passenger\s+    # matches the characters Passenger literally (case sensitive) follow by more than one white space
    (Info|Details)  # match the characters Info or Details literally (case sensitive)
    $               # assert position at end of the string
    """)

# Regex to match operating airline from string eg: DELTA 243
OPERATING_AIR_LINE1 = re.compile(r"""
    (?x)            # verbose regex
    (
        [A-Z]+     # match more than one character from range [A-z]
        \ *        # match zero or more white space character
        [A-Z]*     # match zero or more character from range [A-Z]
    )
    (?:\s\d{2,4})  # Non-capturing group (match white space character follow by two to four digits)
    """)

# To match flight reward program name like 'SkyMiles'
REWARD_PROGRAM = re.compile(r"""
    (?x)      # verbose regex
    (\s*SkyMiles)   # match word SkyMiles
    """)

LOYALTY_REGEX = re.compile(r"""
    (?x)      # verbose regex
    (?:\s*SkyMiles\s*\#) # Non-capturing group (match word SkyMiles)
    ([\*A-z0-9]+)  # match character * and list [A-z] and [0-9]
    """)

OPERATING_FLIGHT_NUMBER = '(?:Flight %s Operated by )(?:.*) (\d+)'

OPERATING_AIR_LINE = '(?:Flight %s Operated by )([A-z ]+)'

DELTA_RULE = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': True, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'operating_airline': [{'rule': OPERATING_AIR_LINE1, 'is_repeated': False, 'is_replace': False},
                          {'rule': OPERATING_AIR_LINE, 'is_repeated': False, 'is_replace': True}],
    'operating_airline_flight_number': {'rule': OPERATING_FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': True, 'is_replace': True},
    'seat_class': {'rule': FLIGHT_CLASS_REGEX, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGER_RULES = {'name': {'rule': PASSENGER_NAME_PATTERN, 'is_repeated': True, 'is_replace': True},
                   'rewards_program': {'rule': REWARD_PROGRAM, 'is_repeated': False, 'is_replace': False},
                   'loyalty': {'rule': LOYALTY_REGEX, 'is_repeated': False, 'is_replace': True},
                   'is_expansion_needed': is_more_expansion_needed}

SEGMENT_IDENTIFY_RULES = [[{'rule': TIME_PATTERN, 'segment_rule': DELTA_RULE, 'is_validate': True}],
                          [{'rule': PASSENGER_IDENTIFY_RULE, 'segment_rule': PASSENGER_RULES, 'is_shift': False}]]


# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
DELTA_RULES_ORDER = ['time', 'operating_airline', 'flight_date', 'flight_number', 'seat_class', 'airport',
                     'name', 'rewards_program', 'loyalty']
