"""
This file contains regular expression  that are using in KLM airlines filter class
"""
import re
from lola.itinerary_parsing.parsers.email_parsers.rules.airlines import is_more_expansion_needed_segment

# To get flight class with regex eg: Class:	Economy
FLIGHT_CLASS_REGEX = re.compile(r"""
   (?x)   # verbose regex
    (?:   # non capture
     Class:\s*  # match word 'Class' follow by zero or more white space
    )
    (.*)  # matches any character (except newline)
    """)

# To get passenger information table eg : Ms Leandra Bell (adult)
PASSENGER_NAME_REGEX = re.compile(r"""
   (?x)   # verbose regex
    (?:Ms|Mr)   # non capture (match word Ms or Mr)
    \s+         # match one or more white space
    ([A-z|\s]+) # match character from range A to Z and a to z or white space
    \s+\(       # match zero ore moe white space following by character (
    """)

# To get all emails that match with KLM airline subject regex
KLM_AIRLINE_SUBJECT_REGEX = r'.*Confirmation:\s+[A-Za-z|\s|-]+\([A-Z]+\)'

DATE_FORMAT = ['%a %d %b %y']


def is_expansion_needed_passenger(flight_info, is_single):
    return not 'name' in flight_info

AIRPORT_NAME = re.compile(r"""(?x) # Example: Amsterdam (Schiphol, Netherlands)
                                 (
                                   [A-z ]+\s*
                                   \(
                                   [A-z ,\-\.\\]+
                                   \)
                                  )
                                """)

FLIGHT_NUMBER = re.compile(r"""(?x) # Example: Flight number: KL 1829
                                 Flight\ +number:\ *([A-Z \d]+)
                                """)

OPERATING_AIR_LINE = re.compile(r"""(?x) # Example: Operated by: KLM
                                 Operated\ +by:\ *([A-z ]+)
                                """)

DATE_PATTERN = re.compile(r"""(?x)([A-z]{3}\ +\d{1,2}\ +[A-z]{3}\ +\d{2})""")

PLAN_TYPE = re.compile(r"""(?x) # Example: Aircraft type: 	Boeing 747-400

                            Aircraft\ +type\ *:\ *
                            (
                                .*\d
                            )

                       """)

SEAT_CLASS = re.compile(r"""(?x) # Example: Class: Economy
                           Class\ *:\ *
                           (
                            [A-z,\-\.\\]+\ *[A-z,\-\.\\]*
                           )
                       """)

TIME_PATTERN = re.compile(r"""(?x)(\d{1,2}\s*\:\s*\d{2})""")

KLM_RULES = {
    'airport': {'rule': AIRPORT_NAME, 'is_repeated': True, 'is_replace': True},
    'flight_number': {'rule': FLIGHT_NUMBER, 'is_repeated': False, 'is_replace': True},
    'operating_airline': {'rule': OPERATING_AIR_LINE, 'is_repeated': False, 'is_replace': True},
    'plane_type': {'rule': PLAN_TYPE, 'is_repeated': False, 'is_replace': True},
    'flight_date': {'rule': DATE_PATTERN, 'is_repeated': False, 'is_replace': True},
    'time': {'rule': TIME_PATTERN, 'is_repeated': False, 'is_replace': True},
    'seat_class': {'rule': FLIGHT_CLASS_REGEX, 'is_repeated': False, 'is_replace': True},
    'is_expansion_needed': is_more_expansion_needed_segment
}

PASSENGERS_RULE = {
    'name': {'rule': PASSENGER_NAME_REGEX, 'is_repeated': True, 'is_replace': True},
    'is_expansion_needed': is_expansion_needed_passenger
}

PASSENGER_SEGMENT_IDENTIFIER = re.compile(r"""Passengers""")

SEGMENT_IDENTIFY_RULES = [[{'rule': DATE_PATTERN, 'segment_rule': KLM_RULES, 'is_validate': True}
                           ], [{'rule': PASSENGER_SEGMENT_IDENTIFIER, 'segment_rule': PASSENGERS_RULE,
                                'is_validate': False}, ]]

# It is important to maintain rules order for the airlines.
# It is an extra step to make sure rules must not conflict with any other information of the flight.
# When any of the info matches to rule than we replace matched text with an empty string
# to be on safe side for the next search.
KLM_RULE_ORDER = ['name', 'airport', 'flight_date', 'time', 'seat_class', 'operating_airline', 'flight_number',
                  'plane_type']
