from __future__ import absolute_import

import json
import logging

import requests
from injector import singleton, inject
from lola.utils.config import LolaEnv
from redis import Redis

from lola.itinerary_parsing.parsers import LolaService
from lola.itinerary_parsing.parsers.city_lookup_service import CityLookupService

logger = logging.getLogger('lola.suggestions')


@singleton
class HotelLookupService(LolaService):
    @inject(redis=Redis, config=LolaEnv,
            city_service=CityLookupService)
    def __init__(self, redis, config, city_service):
        super(HotelLookupService, self).__init__()
        self.config = config
        self.redis = redis
        self.city_service = city_service

    def get_star_rating(self, hotel_name, hotel_address):
        response = self.lookup_hotel_on_google(hotel_name, hotel_address)
        if not response:
            return None
        data = json.loads(response)
        results = data.get('results', None)
        result = None
        if results:
            result = results[0]
        rating = None
        if result:
            rating = result.get('rating')
        return rating

    def lookup_city(self, hotel_name, hotel_address):
        response = self.lookup_hotel_on_google(hotel_name, hotel_address)
        if not response:
            return None
        data = json.loads(response)
        results = data.get('results', None)
        result = None
        if results:
            result = results[0]
            # really we should take a look to see if there is already geo data in here
            address = result.get('formatted_address')
            if address:
                return self.city_service.parse_city(address)
        return {}

    def lookup_hotel_on_google(self, hotel_name, hotel_address):
        hotel_name = hotel_name or ""
        hotel_address = hotel_address or ""
        print "looking up %s on google" % hotel_name
        key = 'hotel_service:google:%s:%s' % (hotel_name, hotel_address)
        google_types = ['lodging']

        if self.redis.exists(key):
            return self.redis.get(key)

        google_api_key = self.config.GOOGLE_API_KEY

        query = hotel_name + " " + hotel_address

        response = requests.get('https://maps.googleapis.com/maps/api/place/textsearch/json', params={
            'query': query,
            'key': google_api_key,
            'types': "|".join(google_types)
        })
        if response.status_code == 200:
            print "   ... got an answer for %s" % hotel_name

            text = response.text
            self.redis.setex(key, text, 1209600)  # two weeks
            return text
        return None

    def parse_address(self, address):
        if not address:
            return {'coordinates': {'lng': None, 'lat': None},
                    'address': {'province': None, 'city': None, 'post_code': None, 'country_code': None}}
        response = self.lookup_address_on_google(address)
        if not response:
            return None
        data = json.loads(response)
        results = data.get('results')
        if not results:
            return {'coordinates': {'lng': None, 'lat': None},
                    'address': {'province': None, 'city': None, 'post_code': None, 'country_code': None}}

        result = results[0]

        address_components = result.get('address_components')
        city = None
        province = None
        postal_code = None
        country_code = None

        if address_components:
            for item in address_components:
                if 'administrative_area_level_1' in item.get('types'):
                    province = item.get('short_name')
                elif 'locality' in item.get('types'):
                    city = item.get('short_name')
                elif 'postal_code' in item.get('types'):
                    postal_code = item.get('short_name')
                elif 'country' in item.get('types'):
                    country_code = item.get('short_name')

        lng, lat = None, None
        try:
            coordinates = result.get('geometry').get('location')
            if coordinates:
                try:
                    lng = float(coordinates.get('lng'))
                    lat = float(coordinates.get('lat'))
                except ValueError, e:
                    logger.exception('Error while casting longitude and latitude for address %s: %s' % address,
                                     e.message)
                    lng = None
                    lat = None
        except KeyError, e:
            logger.exception('Error while getting longitude and latitude for address %s: %s' % address, e.message)
            lng = None
            lat = None

        return {'coordinates': {'lng': lng, 'lat': lat}, 'address':
            {'province': province, 'city': city, 'post_code': postal_code, 'country_code': country_code}}

    def lookup_address_on_google(self, address):
        key = 'city_service:google:%s' % address

        if self.redis.exists(key):
            return self.redis.get(key)

        google_api_key = self.config.GOOGLE_API_KEY

        response = requests.get('https://maps.googleapis.com/maps/api/geocode/json', params={
            'address': address,
            'key': google_api_key
        })

        if response.status_code == 200:
            text = response.text
            self.redis.setex(key, text, 1209600)  # two weeks
            return text
        return None
