from __future__ import absolute_import
import copy

import logging
import re
import time

from dateutil.parser import parse

from lola.utils.injection import injector
from lola.itinerary_parsing.gmail import authenticate, get_email_from_token
from lola.itinerary_parsing.parsers.api_parsers.tripit_api_parser import TripItApiParser
# from lola.itinerary_parsing.parsers.email_parsers import TRAVEL_FILTERS, lookup_filter_class
from lola.itinerary_parsing.parsers.email_parsers.hotels import *
from lola.itinerary_parsing.parsers.email_parsers.airlines import *
from lola.itinerary_parsing.parsers.email_parsers import *
from lola.itinerary_parsing.utils import TravelEmail
from lola.itinerary_parsing.utils import decode_str
from lola.itinerary_parsing.validation_json_schema.itinerary_validator import ItineraryValidator

logger = logging.getLogger('lola.itinerary_parser')


class ItineraryParsing():
    def __init__(self, access_tokens, prop=None):
        if prop is None:
            prop = {}
        self.access_tokens = access_tokens
        self.prop = prop
        self.email = None
        self.source = None

    def scan_parse_travel_emails(self):
        self.email = get_email_from_token(self.access_tokens.get('access_token'))
        self.source = 'Gmail'

        gmail_obj = authenticate(self.email, self.access_tokens.get('access_token'))
        num_filters = len(TRAVEL_FILTERS)

        for index, pattern in enumerate(TRAVEL_FILTERS):
            progress = (float(index + 1) / num_filters) * 100
            message_list = gmail_obj.all_mail().mail(subject=pattern['subject'], fr=pattern['sender'], prefetch=True)
            for message in message_list:
                travel_email = {}
                try:
                    status = 'success'
                    error_msg = ''
                    body = message.body or ''
                    html = message.html
                    message_id = message.message_id
                    sent_at = message.sent_at
                    subject = message.subject
                    sender = message.fr
                    thread = message.thread_id
                    regex_list = pattern['regex']

                    if regex_list:
                        for regex in regex_list:
                            pat = re.compile(regex)
                            if pat.match(subject):
                                break
                        else:
                            continue

                    html = decode_str(html)
                    body = decode_str(body)
                    filtered_subj = unicode(subject)
                    logger.info('scanned email for source: %s' % pattern['source'])
                    travel_email = {'body': body, 'source': pattern['source'],
                                    'html': html, 'message_id': message_id, 'sent_at': sent_at,
                                    'subject': subject, 'sender': sender, 'thread': thread,
                                    'filtered_subject': ' '.join(map(lambda x: x.strip(), re.split(r'\r\n',
                                                                                                   filtered_subj)))
                                    }
                    annotation = self.parse_specific_email(TravelEmail(travel_email))
                    if not annotation.get('raw_email'):
                        annotation['raw_email'] = travel_email

                    annotation['itinerary_id'] = self.create_unique_email_identifier(annotation, message)

                except Exception as error:
                    annotation = {'raw_email': travel_email}
                    status = 'failure'
                    error_msg = error.message
                    logger.exception('Exception while scanning travel email %s', error.message)

                yield self.create_response(progress=progress, status=status, error_msg=error_msg, annotation=annotation)

            yield self.create_response(progress, validate=False)

    def create_response(self, progress, status='success', error_msg='', annotation=None, validate=True):
        if not annotation:
            annotation = {}

        if not error_msg and validate:
            validator = ItineraryValidator(copy.deepcopy(annotation))
            validation_output = validator.validate()
            if validation_output['error']:
                status = 'failure'
                error_msg = validation_output['error_msg']

        return {
            'status': status,
            'message': error_msg,
            'itinerary': annotation,
            'meta': {
                'user': self.email,
                'source': self.source,
                'progress': round(progress, 2)
            }
        }

    def parse_travel_emails(self, emails, dry_run=False):
        """
        Parse travel email and then save output in Database
        """
        annotations = []
        for email in emails:
            email = TravelEmail(email)
            travel_filter = injector.get(lookup_filter_class(email.source))
            try:
                annotations.append(travel_filter.parse_message(email))
            except Exception, error:
                logger.exception('Exception while parsing travel email %s', error.message)
            return annotations

    def create_unique_email_identifier(self, annotation, message):
        transaction_id = annotation.get('transaction_id')
        if isinstance(transaction_id, list):
            transaction_id = '-'.join(transaction_id)
        if message.message_id:
            return '%s-%s' % (transaction_id, message.message_id)

        sent_at = parse(message.sent_at)
        sent_at = int(time.mktime(sent_at.timetuple()))
        return '%s-%s' % (transaction_id, sent_at)

    def crawl_tripit_api(self):
        t = TripItApiParser(self.access_tokens)
        self.email = t.email
        self.source = 'Tripit'

        trip_ids = t.get_trip_ids()
        remaining_trips = len(trip_ids)
        num_trips = len(trip_ids)
        logger.info('%s trip(s) found for user: %s!' % (remaining_trips, self.email))

        for index, t_id in enumerate(trip_ids):
            progress = (float(index + 1) / num_trips) * 100
            status = 'success'
            error_msg = ''
            try:
                annotations = t.parse_trip(t_id)
            except Exception as error:
                annotations = [{}]
                status = 'failure'
                error_msg = error.message
                logger.exception(error.message)

            for annotation in annotations:
                yield self.create_response(progress=progress, status=status, error_msg=error_msg, annotation=annotation)

            remaining_trips -= 1
            if remaining_trips:
                logger.info('%s trip(s) remaining.' % remaining_trips)
            else:
                logger.info('Trips parsing completed.')

    def crawl_travel_api(self):
        source = self.prop.get('source', '').lower()
        if source == 'tripit':
            return self.crawl_tripit_api()

    def parse_specific_email(self, email):
        """
        This function is used only in debugging and testing email parsing
        """
        source = email.source
        travel_filter = injector.get(lookup_filter_class(source))
        annotations = travel_filter.parse_message(email)
        return annotations
