"""
This file contain globally used function in hotel and airline filter class
"""
# -*- coding: UTF-8 -*-
from __future__ import absolute_import
from datetime import datetime
import re
import logging
import unicodedata

from dateutil import parser
from bs4 import NavigableString, Tag, Comment
from injector import Key
from address import AddressParser

from lola.itinerary_parsing.utils.currency_code_names import currencies
from lola.itinerary_parsing.utils.tripit import TripIt, OAuthConsumerCredential

logger = logging.getLogger('lola.itinerary_parsing.utils')

ImportName = Key('ImportName')
ILLEGAL_TOKENS = ['(', ')', 'at', '-']
PATTERN_HEADER_TEXT = re.compile(r"""
                                    (?x)    # Sample matched pattern: Arrival Date:
                                    (.*)    # Capture everything till we
                                    (:)$    # have a semicolon at the end of the string
                                    """)


def strip_unicode(string):
    """
    Function to strip unicode from given string
    """
    response = unicodedata.normalize('NFKD', string).encode('ASCII', 'ignore')
    return response


def find_date_in_string(date_string, default=datetime.now().date()):
    """
    Find a date from given string
    """
    # this is a fancy heuristic that tries to find anything that looks like a date in a string
    # the contract is also that it never returns a date that is earlier than the default
    # so if today is June 3rd, 2015 and the string contains "June 1", it will return June 1, 2016

    date_components = date_string.split()
    date_string = ' '.join([item for item in date_components if item not in ILLEGAL_TOKENS])
    # todo: perhaps we should do this higher up on all strings??
    if isinstance(date_string, unicode):
        date_string = strip_unicode(date_string)

    def parse_date(string):
        """
        Parse date from string
        """
        result = parser.parse(string, default=datetime(1581, 1, 1))
        if result.year == 1581:
            result = result.replace(year=default.year)
            # if result.date() < default:
            #     result = result.replace(year=default.year + 1)
        return result

    def date_candidate(string):
        # don't bother parsing very short strings.  The shortest date
        # we could possibly get is "1/1".
        # maybe there will be other tests?
        return len(string) > 2

    def shrink_string_from_right(string):
        """
        Function to shrink any string from right to left
        """

        # shrinking the string should work like this:
        # look at the last character of the string and then trim all characters of that type from the right
        # i.e., trim strings of letters or numbers but not both.  In addition, we might need to do this at both
        # ends at the same time...
        # but for now, just shrink it by 1
        return string[0:-1]

    def shrink_string_from_left(string):
        """
        Function to shrink given string from left
        """
        return string[1:]

    methods = [shrink_string_from_right, shrink_string_from_left]

    for shrinker in methods:
        candidate = date_string
        while candidate:
            try:
                d = parse_date(candidate)
                return d
            except ValueError:
                # if we couldn't parse the string, try shrinking it a bit.
                candidate = shrinker(candidate)
                if not date_candidate(candidate):
                    break
    return None


def node_contents(content):
    # there might be cells that are badly structured, so we need to check to see if this specific
    # cell has some direct cell children of its own (common mistake).
    contents = ""
    if not content:
        return contents
    for child in content.children:
        if contents:
            contents += " "
        if isinstance(child, NavigableString):
            try:
                contents += unicode(child)
            except:
                pass
        else:
            contents += child.text
    return contents.strip()


def parse_amount(amount_str):
    """
    This function parse amount out of string
    """
    # Removes any currency symbol or code from string
    amount = 0
    amount_regex = re.compile(r'(\d+[,\d{3}]*\.?\d{,2})')
    amount_result = amount_regex.search(amount_str)
    if amount_result:
        amount = str(amount_result.group(1))
        amount = re.sub(r',', '', amount)
    return amount


def remove_empty_childs(parent):
    """
    Remove empty child nodes from list
    """
    if not parent or isinstance(parent, NavigableString) or isinstance(parent, Comment):
        return

    loop_continue = True
    while loop_continue:
        loop_continue = False
        for child in parent.contents:
            if isinstance(child, NavigableString):
                if not child or child.isspace() or re.sub(r'\n+', ' ', child).isspace() or isinstance(child, Comment):
                    child.extract()
                    loop_continue = True
                    break
            else:
                if (not child.text or child.text.isspace() or re.sub(r'\n+', ' ', child.text).isspace()) and \
                        not child.find('img') and child.name not in ['img', 'br']:
                    child.extract()
                    loop_continue = True
                    break


def remove_empty_childs_airlines(parent):
    """
    Remove empty child nodes from list
    """
    loop_continue = True
    while loop_continue:
        loop_continue = False
        try:
            for child in parent.contents:
                if isinstance(child, NavigableString):
                    if not child or child.isspace() or re.sub(r'\n', ' ', child).isspace() or isinstance(child, Comment):
                        child.extract()
                        loop_continue = True
                        break
                else:
                    if (not child.text or child.text.isspace() or re.sub(r'\n', ' ', child.text).isspace()) and \
                            (not child.name in ['td', 'tr'] or child.find('img')):
                        child.extract()
                        loop_continue = True
                        break
        except (TypeError, AttributeError):
            pass


def parse_email_information(email):
    """
    Return required emails information in required fields
    """
    pattern = re.compile(r'(?:@)(.*)')
    match = pattern.findall(email.sender)
    email_information = {'source': str(match[0]) if match else None,
                         'subject': str(email.subject) if email.subject else None,
                         'time': convert_datetime_to_timestamp(str(email.sent_at)) if email.sent_at else None,
                         'body': convert_to_str(email.body) if email.body else convert_to_str(email.html),
                         'attachments': None
                         }
    return email_information


def convert_datetime_to_timestamp(date_time):
    """
    Convert time into timestamp format
    """
    try:
        return parser.parse(str(date_time)).isoformat()
    except ValueError:
        pass
    return None


def convert_to_str(string):
    """
    Encode string into utf-8 format
    """
    if string:
        result = string.encode('utf-8').strip()
        return result


def find_target(soup, regex):
    """
    Find target table of a row/column with some regex/text
    """
    return get_parent_table(soup.find(text=regex))


def get_parent_table(target):
    """
    To get parent table of a target (target can be any node  like row/column)
    """
    while target and (isinstance(target, NavigableString) or target.name != 'table'):
        target = target.parent
    return target


def calculate_airtime(departure, arrival):
    """
    Function to calculate flight airtime in seconds
    """
    if not (arrival and departure):
        return 0

    time_difference = arrival - departure
    return time_difference.total_seconds()


def extract_info_from_address(text):
    """
    Extract info out of address e.g State, Zip code
    Country code, City name
    """
    address = AddressParser()
    return address.parse_address(text)


def is_table_header(parent_node):
    """
    Check either if given row represents header row or not
    """
    valid_node_count = 0
    decision = False
    for child in parent_node.contents:
        if not isinstance(child, NavigableString):
            if child.name == 'th':
                valid_node_count += 1
                decision = True
            elif child.text and re.search(PATTERN_HEADER_TEXT, child.text):
                valid_node_count += 1
                decision = True
            elif not child.text or child.text.isspace():
                continue
            else:
                decision = False
                break
    if valid_node_count == 1:
        decision = False
    return decision


def is_empty_node(node):
    """
    checks if node is empty
    """
    decision = False
    if isinstance(node, NavigableString):
        if node.isspace():
            decision = True
        elif not node:
            decision = True
    return decision


def is_valid_node(node):
    """
    Checks if node is valid according to
    rules defined
    """
    decision = True
    if isinstance(node, Tag):
        if node.text.isspace():
            decision = False
        elif not node.text:
            decision = False
        elif node.name == 'li':
            decision = False
    return decision


def find_values(soup, regex):
    """
    To find multiple values in soup through some regex pattern
    """
    values = []
    for target in regex.finditer(soup.text):
        if target:
            values.append(convert_to_str(target.group(1)))
    return values


def merge_child_nodes(node, element_position):
    """
    Make an analysis of child node text on the behalf of the ':' token.
    Purpose of this method is to merge child nodes of a given node.
    Check if any node contains all text in it or it could be collected
    from any other node, if this is the case then merge multiples node into
    a single node.
    """
    children = []
    collected_text = u''
    traverse_count = -1

    if isinstance(node, NavigableString):
        return children

    for child in node.contents:
        if isinstance(child, Tag):
            child = child.text

        if child.endswith(':') and collected_text:
            collected_text = '%s %s' % (collected_text, child)
            children.append(collected_text)
            collected_text = u''
            traverse_count += 1
        elif child.endswith(':'):
            children.append(child)
            collected_text = u''
            traverse_count += 1
        else:
            collected_text = '%s %s' % (collected_text, child)
        if traverse_count == element_position:
            break
    return children


def map_cabin_type(cabin_name):
    """
    Lookup flight cabin type (seat class)
    """
    cabin_types = [('ECONOMY', 'economy', 'b(coach)', 'e(coach)', 'h(coach)', 'k(coach)', 'world traveller', 'domestic',
                    'l(coach)', 'm(coach)', 'n(coach)', 'q(coach)', 's(coach)', 't(coach)',
                    'v(coach)', 'w(coach)', 'x(coach)', 'y(coach)', 'world traveller', 'basic economy ',
                    'main cabin', 'economy (t)', 't', 'k', 'coach', 'x', 'y', 'i', 'b', 'h', 'l', 'w', 's', 'n', 'u', 'v'),
                   ('PREMIUM_ECONOMY', 'premium economy', 'international economy', 'world traveller plus', 'prem plus',
                    'w'),
                   ('FIRST', 'first', 'a(coach)', 'f(coach)', 'p(coach)', 'r(coach)', 'first cl', 'discount', 'f', 'o'
                    , 'a', 'p'),
                   ('BUSINESS', 'international business', 'north america business', 'business', 'j', 'r', 'd'
                                                                                                          'c(coach)',
                    'd(coach)', 'h(coach)', 'z(coach)', 'club europe', 'biz bed', 'club world')
                   ]
    for cabin_type_tuple in cabin_types:
        for name in cabin_type_tuple[1:]:
            if name == cabin_name.lower():
                return cabin_type_tuple[0]
    return None


def remove_element_from_soup(elements):
    for element in elements:
        element.extract()


def get_node_position_under_parent(parent, element):
    """
    This function helps to find the index of an element
    with reference to parent element. If there exist an element
    which is not a direct child of parent element so this
    function tries to reach an element which is direct child of parent
    in order to determine position of element under parent.
    :param parent: Parent Element
    :param element: Element whose position has to be
    determine under parent
    :return: Return int value
    """
    try:
        while element.parent:
            if parent != element.parent:
                element = element.parent
                continue
            try:
                return parent.index(element), element
            except ValueError:
                element = element.parent
    except AttributeError:
        pass
    return -1, None


def parse_leaves(row):
    """
    parse child's tables if any present in rows
    """
    if row:
        for child in row.children:
            if isinstance(child, NavigableString):
                pass
            elif child.name in ['td', 'th']:
            # recurse into this child's tables if any, otherwise yield self
                something = False
                for cell_child in child.children:
                    if cell_child.name == 'table':
                        something = True
                        for cell in parse_leaves(cell_child):
                            yield cell
                    elif cell_child.name == 'div':
                        something = True
                        yield cell_child
                if not something:
                    yield child
            elif child.name in ['table', 'tbody', 'tr']:
                for inner_cell in parse_leaves(child):
                    yield inner_cell


def get_node_contents(node_row):
    """
    Parse row(its child table structure) contents
    """
    contents = []
    for leaf in parse_leaves(node_row):
        content = leaf.text.strip()
        if content and content != 'N/A':
            contents.append(strip_unicode(content))
    return contents


def get_next_sibling_target(target, is_parent):
    """
    To get next sibling target by giving some target node
    """
    while target:
        if target.next_sibling:
            if isinstance(target.next_sibling, NavigableString) or not target.next_sibling.text.strip():
                target = target.next_sibling
                continue
            return target.next_sibling

        if not is_parent:
            break
        target = target.parent


def get_element_common_parent(element, text, is_single_segment=False):
    """
    Get element comment parent by matching text
    Purpose of getting common parent because it help avoiding badly structured
    html.
    """
    text1 = element
    text = unicode(text)
    text_pattern = re.compile('^' + replace_regex_characters(text) + '$')
    element = element.parent

    while element.parent:
        if not element.find(text=text_pattern):
            element = element.parent
            continue
        break

    # There are cases when text for segment identifying elements are same.
    # This scenario leads us to get wrong element as a common parent.
    # Following piece of code checks if markup for both the element are same,
    # keep on traversing parent until we found pattern count greater than 1.
    # That is where we get common parent in such scenarios.
    if not is_single_segment and text1 == text:
        while len(element.findAll(text=text_pattern)) <= 1 and element.parent:
            element = element.parent
    return element


def standardize_currency_name(currency):
    currency = currency.strip().lower()
    if isinstance(currency, unicode):
        try:
            currency = currency.encode('utf-8')
        except:
            pass
    for k in currencies.keys():
        if k.lower() == currency:
            return k
        if currencies[k]['symbol'].lower() == currency:
            return k
        if currency in [currencies[k]['name'].lower(), currencies[k]['name_plural'].lower()]:
            return k
    return currency


def get_tripit_email(params):
    """
    From Tripit access tokens get
    the email address of the associated
    account
    """
    valid_token = OAuthConsumerCredential(params.get('consumer_key'),
                                          params.get('consumer_secret'),
                                          params.get('access_token'),
                                          params.get('access_secret'))
    tripit_client = TripIt(valid_token)
    profile = tripit_client.get_profile()
    email = ''
    if profile._children:
        current_child = profile._children[0]
    else:
        return email
    for i in xrange(0, 5):
        for child in current_child._children:
            if 'email' in child.__name__.lower():
                child_attrs = child._attributes
                if child_attrs.get('address'):
                    return child_attrs.get('address')
                else:
                    current_child = child
                    break
    return email


def convert_to_desired_val(amount, amount_type=int, default=0):
    try:
        return amount_type(amount)
    except (ValueError, TypeError):
        return default


def replace_regex_characters(text):
    characters_mapping = {'(': '\(', ')': '\)', '+': '\+'}
    for key, val in characters_mapping.iteritems():
        text = text.replace(key, val)
    return text


def get_matched_text(match):
    matched_text = []
    for item in match:
        if isinstance(item, tuple):
            matched_text.extend([val for val in item if val and not val.isspace()])
            continue
        matched_text.append(item)
    return matched_text


def get_element_text(element, separator=u''):
    return element if isinstance(element, NavigableString) else element.getText(separator=separator)


def decode_str(raw_str):
    if not raw_str:
        return ''
    try:
        return raw_str.decode('utf-8')  # see if this will pass muster
    except:
        # if not, try decoding magically
        return raw_str.decode('windows-1252')


def overrides(interface_class):
    def overrider(method):
        assert (method.__name__ in dir(interface_class))
        return method

    return overrider


def first(l, exception=None):
    if len(l) > 0:
        for item in l:
            return item
    if exception:
        raise exception()
    return None


def dotget(m, path, default=None):
    result = None
    if m is not None:
        result = reduce(lambda x, y: x.get(y) if isinstance(x, dict) else x, path.split('.'), m)
    if result is None:
        result = default
    return result


class TravelEmail(object):
    def __init__(self, d):
        self.__dict__ = d