"""
This file contains flight validator class and its methods.
"""
import json
import re

from lola.itinerary_parsing.validation_json_schema import BaseValidator
from lola.itinerary_parsing.validation_json_schema.error_msgs import INVALID_AIRPORT_CODES, NO_LEG_ERROR,\
    INVALID_FLIGHT_NUMBER, INVALID_FLIGHT_DATES


class FlightValidator(BaseValidator):
    """
     This class contains flight itinerary validator methods
    """
    def __init__(self, flight):
        self.flight = flight
        self.airport_code_regex = re.compile(r'^[A-Z]{3}$')
        self.max_flight_number_length = 10
        self.errors = []

    def validate(self):
        """
        Validate flight itinerary data
        """
        if not self.contains_valid_list_value(self.flight, 'legs'):
            self.errors.append(NO_LEG_ERROR)
        else:
            self.validate_legs()

        return {'error': len(self.errors) > 0, 'error_msg': json.dumps(self.errors)}

    def validate_legs(self):
        """
        check if flight legs and leg's segments data are valid
        """
        legs = self.flight['legs']
        legs_validated = True

        while legs and legs_validated:
            leg = legs.pop(0)
            self.contains_valid_list_value(leg, 'segments')
            for segment in leg['segments']:
                if not self.is_valid_segment(segment):
                    legs_validated = False
                    break
        return legs_validated

    def is_valid_segment(self, segment):
        """
        Check if given segment data is valid
        """
        is_valid = True
        arrival_airport = segment.get('arrival_airport')
        departure_airport = segment.get('departure_airport')
        departure_at = segment.get('departure')
        arrival_at = segment.get('arrival')
        flight_num = segment.get('flight_number')

        valid_airports = self.is_valid_airport_code(arrival_airport) \
                         and self.is_valid_airport_code(departure_airport)

        if not valid_airports:
            is_valid = False
            self.errors.append(INVALID_AIRPORT_CODES % (arrival_airport, departure_airport))

        valid_dates = self.is_valid_date(departure_at) and self.is_valid_date(arrival_at)
        if not valid_dates:
            is_valid = False
            self.errors.append(INVALID_FLIGHT_DATES % (departure_at, arrival_at))

        valid_flight_num = self.is_valid_flight_number(flight_num)
        if not valid_flight_num:
            is_valid = False
            self.errors.append(INVALID_FLIGHT_NUMBER % flight_num)

        return is_valid

    def is_valid_airport_code(self, airport_code):
        """
        Check if given airport code is passed thorough regex validation
        """
        if not airport_code or not isinstance(airport_code, basestring):
            return False
        if self.airport_code_regex.match(airport_code.strip()):
            return True
        return False

    def is_valid_flight_number(self, flight_number):
        """
        check if given number if valid flight number length
        """
        if not flight_number or not isinstance(flight_number, basestring):
            return False

        if len(flight_number.strip()) > self.max_flight_number_length:
            return False

        return True
