"""
This file contains hotel validator class and its methods.
"""
import json
from lola.itinerary_parsing.validation_json_schema import BaseValidator
from lola.itinerary_parsing.validation_json_schema.error_msgs import INVALID_HOTEL_NAME, INVALID_HOTEL_DATES, \
    INVALID_HOTEL_ADDRESS


class HotelValidator(BaseValidator):
    """
    This class contains hotel itinerary validator methods
    """

    def __init__(self, hotel):
        self.hotel = hotel
        self.min_name_length = 10
        self.max_name_length = 60
        self.address_min_length = 10
        self.errors = []

    def validate(self):
        """
        Validate hotel data
        """
        self.is_valid_hotel_name()
        self.valid_hotel_dates()
        self.validate_address()

        return {'error': len(self.errors) > 0, 'error_msg': json.dumps(self.errors)}

    def is_valid_hotel_name(self):
        """
        Check if hotel name length is valid
        """
        hotel_name = self.hotel.get('name', '')
        if hotel_name and self.max_name_length >= len(hotel_name) >= self.min_name_length:
            return True

        self.errors.append(INVALID_HOTEL_NAME % hotel_name)
        return False

    def valid_hotel_dates(self):
        """
        Check if hotel checkin and checkout dates are valid
        """
        checkin_date = self.hotel.get('checkin_date')
        checkout_date = self.hotel.get('checkout_date')

        if self.is_valid_date(checkin_date) and self.is_valid_date(checkout_date):
            return True

        self.errors.append(INVALID_HOTEL_DATES % (checkin_date, checkout_date))
        return False

    def validate_address(self):
        """
        Check if hotel address length is valid
        """
        address = self.hotel.get('address1', '')
        if address and len(address) >= self.address_min_length:
            return True

        self.errors.append(INVALID_HOTEL_ADDRESS % address)
        return False
